{
    "$graph": [
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 16000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/eclip:0.7.0_python"
                }
            ],
            "baseCommand": [
                "barcodecollapsepe.py"
            ],
            "arguments": [
                "-o",
                "$(inputs.input_barcodecollapsepe_bam.nameroot).rmDup.bam",
                "-m",
                "$(inputs.input_barcodecollapsepe_bam.nameroot).rmDup.metrics"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-b"
                    },
                    "label": "",
                    "doc": "input bam to barcode collapse. NOTE: no use for a bai index file!",
                    "id": "#barcodecollapse_pe.cwl/input_barcodecollapsepe_bam"
                }
            ],
            "doc": "This tool wraps barcodecollapsepe.py, a paired-end PCR duplicate removal script\nwhich reads in a .bam file where the first string left of : split of the read name is the barcode\nand merge reads mapped to the same position that have the same barcode.\nAssumes paired end reads are adjacent in output file (ie needs unsorted bams)\nAlso assumes no multimappers in the bam file (otherwise behavior is undefined)\n  Usage: python barcodecollapsepe.py --bam BAM --out_file OUT_FILE --metrics_file METRICS_FILE\n",
            "id": "#barcodecollapse_pe.cwl",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_barcodecollapsepe_bam.nameroot).rmDup.bam"
                    },
                    "label": "",
                    "doc": "barcode collapseed mappings bam ",
                    "id": "#barcodecollapse_pe.cwl/output_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_barcodecollapsepe_bam.nameroot).rmDup.metrics"
                    },
                    "label": "",
                    "doc": "barcode collapse metrics",
                    "id": "#barcodecollapse_pe.cwl/output_barcodecollapsepe_metrics"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 1000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/eclip:0.7.0_python"
                }
            ],
            "baseCommand": [
                "calculate_entropy.py"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--full"
                    },
                    "label": "",
                    "doc": "output full file from overlap_peakfi_with_bam.pl (should contain number of reads per peak)",
                    "id": "#calculate_entropy.cwl/full"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--input_mapped"
                    },
                    "label": "",
                    "doc": "File containing a single number corresponding to the number of mapped reads in INPUT",
                    "id": "#calculate_entropy.cwl/input_mapped"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--ip_mapped"
                    },
                    "label": "",
                    "doc": "File containing a single number corresponding to the number of mapped reads in IP",
                    "id": "#calculate_entropy.cwl/ip_mapped"
                }
            ],
            "arguments": [
                "--output",
                "$(inputs.full.nameroot).entropynum"
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.full.nameroot).entropynum"
                    },
                    "label": "",
                    "doc": "File containing the sum entropy value",
                    "id": "#calculate_entropy.cwl/output_entropynum"
                }
            ],
            "doc": "This tool computes and sums the entropy values for significant peaks (l10p >=3 and l2fc >=3).\nReturns the number as a file.\n",
            "id": "#calculate_entropy.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 8,
                    "ramMin": 32000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/clipper:5d865bb"
                }
            ],
            "baseCommand": [
                "clipper"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bam"
                    },
                    "id": "#clipper.cwl/bam"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--gene"
                    },
                    "id": "#clipper.cwl/gene"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--outfile",
                        "valueFrom": "${\n  if (inputs.outfile == \"\") {\n    return inputs.bam.nameroot + \".peakClusters.bed\";\n  }\n  else {\n    return inputs.outfile;\n  }\n}\n"
                    },
                    "id": "#clipper.cwl/outfile"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 0,
                        "prefix": "--species"
                    },
                    "doc": "species: one of ce10 ce11 dm3 hg19 GRCh38 mm9 mm10 GRCh38_pU6 GRCh38_v29 GRCh38_v29e hg19_VSV",
                    "id": "#clipper.cwl/species"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.outfile == \"\") {\n    return inputs.bam.nameroot + \".peakClusters.bed\";\n  }\n  else {\n    return inputs.outfile;\n  }\n}\n"
                    },
                    "id": "#clipper.cwl/output_bed"
                }
            ],
            "doc": "CLIPper is a tool to define peaks in your CLIP-seq dataset.\nCLIPper was developed in the Yeo Lab at the University of California, San Diego.\n  Usage: clipper --bam CLIP-seq_reads.srt.bam --species hg19 --outfile CLIP-seq_reads.srt.peaks.bed\n",
            "id": "#clipper.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/eclipdemux:0.0.1"
                }
            ],
            "baseCommand": [
                "eclipdemux"
            ],
            "arguments": [
                "--metrics",
                "$(inputs.dataset).$(inputs.reads.name).---.--.metrics",
                "--expectedbarcodeida",
                "$(inputs.reads.barcodeids[0])",
                "--expectedbarcodeidb",
                "$(inputs.reads.barcodeids[1])"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--barcodesfile"
                    },
                    "id": "#demux_pe.cwl/barcodesfasta"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--dataset"
                    },
                    "id": "#demux_pe.cwl/dataset"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--length"
                    },
                    "doc": "randomer length",
                    "id": "#demux_pe.cwl/randomer_length"
                },
                {
                    "type": {
                        "type": "record",
                        "fields": [
                            {
                                "type": {
                                    "type": "array",
                                    "items": "string"
                                },
                                "name": "#demux_pe.cwl/reads/barcodeids"
                            },
                            {
                                "type": "string",
                                "inputBinding": {
                                    "position": 4,
                                    "prefix": "--newname"
                                },
                                "name": "#demux_pe.cwl/reads/name"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 1,
                                    "prefix": "--fastq_1"
                                },
                                "name": "#demux_pe.cwl/reads/read1"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 2,
                                    "prefix": "--fastq_2"
                                },
                                "name": "#demux_pe.cwl/reads/read2"
                            }
                        ]
                    },
                    "id": "#demux_pe.cwl/reads"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "outputBinding": {
                        "glob": "$(inputs.reads.barcodeids[0])",
                        "loadContents": true,
                        "outputEval": "$(self[0].contents)"
                    },
                    "id": "#demux_pe.cwl/barcodeidA"
                },
                {
                    "type": "string",
                    "outputBinding": {
                        "glob": "$(inputs.reads.barcodeids[1])",
                        "loadContents": true,
                        "outputEval": "$(self[0].contents)"
                    },
                    "id": "#demux_pe.cwl/barcodeidB"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).$(inputs.reads.barcodeids[0]).r1.fq.gz"
                    },
                    "id": "#demux_pe.cwl/demuxedAfwd"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).$(inputs.reads.barcodeids[0]).r2.fq.gz"
                    },
                    "id": "#demux_pe.cwl/demuxedArev"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).$(inputs.reads.barcodeids[1]).r1.fq.gz"
                    },
                    "id": "#demux_pe.cwl/demuxedBfwd"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).$(inputs.reads.barcodeids[1]).r2.fq.gz"
                    },
                    "id": "#demux_pe.cwl/demuxedBrev"
                },
                {
                    "type": "string",
                    "outputBinding": {
                        "glob": "$(inputs.reads.name)",
                        "loadContents": true,
                        "outputEval": "$(self[0].contents)"
                    },
                    "id": "#demux_pe.cwl/name"
                },
                {
                    "type": "string",
                    "outputBinding": {
                        "glob": "$(inputs.dataset)",
                        "loadContents": true,
                        "outputEval": "$(self[0].contents)"
                    },
                    "id": "#demux_pe.cwl/output_dataset"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).---.--.metrics"
                    },
                    "label": "",
                    "doc": "demuxedpairedend metrics",
                    "id": "#demux_pe.cwl/output_demuxedpairedend_metrics"
                }
            ],
            "doc": "demultiplex utility for paired-end eCLIP raw fastq files (process eCLIP barcodes and ramdomers)\nSee: https://github.com/YeoLab/eclipdemux for full code and documentation\n  Usage: eclipdemux --dataset DATASET_ID --metrics METRICS_FILE --fastq_1 READ_1 --fastq_2 READ_2 --expectedbarcodeida BARCODE_A --expectedbarcodeidb BARCODE_B --barcodesfile BARCODES_FASTA --length LENGTH\n",
            "id": "#demux_pe.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 2,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/fastqc:0.11.8"
                }
            ],
            "baseCommand": [
                "fastqc",
                "-t",
                "2",
                "--extract",
                "-k",
                "7"
            ],
            "inputs": [
                {
                    "type": "string",
                    "default": ".",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-o"
                    },
                    "label": "",
                    "doc": "",
                    "id": "#fastqc.cwl/output_postfix"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "label": "",
                    "doc": "",
                    "id": "#fastqc.cwl/reads"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*/fastqc_report.html"
                    },
                    "id": "#fastqc.cwl/output_qc_report"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*/fastqc_data.txt"
                    },
                    "id": "#fastqc.cwl/output_qc_stats"
                }
            ],
            "id": "#fastqc.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/fastq-tools:0.8"
                }
            ],
            "baseCommand": [
                "fastq-sort"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--id"
                    },
                    "label": "",
                    "doc": "input fastq",
                    "id": "#fastqsort.cwl/input_fastqsort_fastq"
                },
                {
                    "type": "string",
                    "default": "",
                    "id": "#fastqsort.cwl/output_fastqsort_fastq"
                }
            ],
            "stdout": "${ if (inputs.output_fastqsort_fastq == \"\") { return inputs.input_fastqsort_fastq.nameroot + \".sorted.fq\"; } else { return inputs.output_fastqsort_fastq; } }",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_fastqsort_fastq == \"\") {\n    return inputs.input_fastqsort_fastq.nameroot + \".sorted.fq\";\n  }\n  else {\n    return inputs.output_fastqsort_fastq;\n  }\n}\n"
                    },
                    "label": "",
                    "doc": "sorted fastq",
                    "id": "#fastqsort.cwl/output_fastqsort_sortedfastq"
                }
            ],
            "doc": "Sorts FASTQ files by their read name. Sorted fastq files are required to keep mapping steps\ndeterministic.\n\n  Usage: fastq-sort --id FASTQ_FILE > STDOUT\n",
            "id": "#fastqsort.cwl"
        },
        {
            "class": "ExpressionTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "loadContents": true
                    },
                    "id": "#file2string.cwl/file"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "id": "#file2string.cwl/output"
                }
            ],
            "expression": "${return {'output':inputs.file.contents}; }",
            "doc": "Returns string expression based on file contents.",
            "id": "#file2string.cwl"
        },
        {
            "class": "ExpressionTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "loadContents": true
                    },
                    "id": "#file2stringArray.cwl/file"
                }
            ],
            "outputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "id": "#file2stringArray.cwl/output"
                }
            ],
            "expression": "${var lines=inputs.file.contents.split('\\n'); var seqs = []; for(var line = 0; line < lines.length; line++) { if(lines[line][0] != '>') { if (!lines[line] || 0 === lines[line].length) {\n} else { seqs.push(lines[line]); } } } return {'output':seqs}; }",
            "doc": "Returns string array expression based on lines in a fasta file (SKIPS >).",
            "id": "#file2stringArray.cwl"
        },
        {
            "class": "CommandLineTool",
            "baseCommand": [
                "gzip"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "id": "#gzip.cwl/input"
                },
                {
                    "type": "boolean",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "default": true,
                    "id": "#gzip.cwl/stdout"
                }
            ],
            "stdout": "$(inputs.input.basename).gz",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input.basename).gz"
                    },
                    "id": "#gzip.cwl/gzipped"
                }
            ],
            "id": "#gzip.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entry": "$(inputs.bam)",
                            "writable": true
                        }
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/makebigwigfiles:0.0.3"
                }
            ],
            "baseCommand": [
                "makebigwigfiles"
            ],
            "arguments": [
                "--bw_pos",
                "$(inputs.bam.nameroot).norm.pos.bw",
                "--bw_neg",
                "$(inputs.bam.nameroot).norm.neg.bw"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bam"
                    },
                    "id": "#makebigwigfiles_PE.cwl/bam"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--genome"
                    },
                    "id": "#makebigwigfiles_PE.cwl/chromsizes"
                },
                {
                    "default": "r",
                    "type": "string",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--direction"
                    },
                    "id": "#makebigwigfiles_PE.cwl/direction"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bam.nameroot).norm.neg.bw"
                    },
                    "id": "#makebigwigfiles_PE.cwl/negbw"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bam.nameroot).norm.pos.bw"
                    },
                    "id": "#makebigwigfiles_PE.cwl/posbw"
                }
            ],
            "doc": "Creates strand-specific bigwig files from a BAM file.\nSee original script here: https://github.com/YeoLab/gscripts/blob/master/gscripts/general/make_bigwig_files_pe.py\n  Usage: makebigwigfiles --bam BAM --genome GENOME --dont_flip --bw_pos --bw_neg\n",
            "id": "#makebigwigfiles_PE.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 16000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/samtools:1.6"
                }
            ],
            "baseCommand": [
                "samtools",
                "sort"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3
                    },
                    "label": "",
                    "doc": "input bam",
                    "id": "#namesort.cwl/input_sort_bam"
                },
                {
                    "type": "boolean",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-n"
                    },
                    "default": true,
                    "id": "#namesort.cwl/name_sort"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-o",
                        "valueFrom": "${\n  if (inputs.output_file == \"\") {\n    return inputs.input_sort_bam.nameroot + \"So.bam\";\n  }\n  else {\n    return inputs.output_file;\n  }\n}\n"
                    },
                    "default": "",
                    "id": "#namesort.cwl/output_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_file == \"\") {\n    return inputs.input_sort_bam.nameroot + \"So.bam\";\n  }\n  else {\n    return inputs.output_file;\n  }\n}\n"
                    },
                    "label": "",
                    "doc": "sorted bam",
                    "id": "#namesort.cwl/output_sort_bam"
                }
            ],
            "doc": "This tool wraps samtools sort, setting the by-name (-n) flag to be True by default.\n  Usage: samtools sort -n <input.bam> <output.bam>\n",
            "id": "#namesort.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/eclip:0.7.0_perl"
                }
            ],
            "baseCommand": [
                "overlap_peakfi_with_bam_PE.pl"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -5
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/clipBamFile"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -2
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/clipReadnum"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -4
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/inputBamFile"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -1
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/inputReadnum"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 0,
                        "valueFrom": "${\n  if (inputs.outputFile == \"\") {\n    return inputs.peakFile.nameroot + \".normed.bed\";\n  }\n  else {\n    return inputs.outputFile;\n  }\n}\n"
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/outputFile"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -3
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/peakFile"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.outputFile == \"\") {\n    return inputs.peakFile.nameroot + \".normed.bed\";\n  }\n  else {\n    return inputs.outputFile;\n  }\n}\n"
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/inputnormedBed"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.outputFile == \"\") {\n    return inputs.peakFile.nameroot + \".normed.bed.full\";\n  }\n  else {\n    return inputs.outputFile;\n  }\n}\n"
                    },
                    "id": "#overlap_peakfi_with_bam_PE.cwl/inputnormedBedfull"
                }
            ],
            "doc": "This tool wraps overlap_peakfi_with_bam_PE.pl\n  Usage:\n",
            "id": "#overlap_peakfi_with_bam_PE.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 1000
                }
            ],
            "baseCommand": [
                "parsebarcodes.sh"
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/eclip:0.7.0_perl"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 3
                    },
                    "id": "#parsebarcodes.cwl/barcodeidA"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#parsebarcodes.cwl/barcodeidB"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "id": "#parsebarcodes.cwl/barcodesfasta"
                },
                {
                    "type": "string",
                    "default": "10",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "randomer length: now normally 10, some old experiment used 5",
                    "id": "#parsebarcodes.cwl/randomer_length"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "A_adapters.fasta"
                    },
                    "id": "#parsebarcodes.cwl/A_adapters"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "a_adapters.fasta"
                    },
                    "id": "#parsebarcodes.cwl/a_adapters"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "a_adapters_default.fasta"
                    },
                    "id": "#parsebarcodes.cwl/a_adapters_default"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "g_adapters.fasta"
                    },
                    "id": "#parsebarcodes.cwl/g_adapters"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "g_adapters_default.fasta"
                    },
                    "id": "#parsebarcodes.cwl/g_adapters_default"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "trimagain_overlap_length.txt"
                    },
                    "id": "#parsebarcodes.cwl/trimagain_overlap_length"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "trimfirst_overlap_length.txt"
                    },
                    "id": "#parsebarcodes.cwl/trimfirst_overlap_length"
                }
            ],
            "doc": "This tool wraps parsebarcodes.sh.\n\nWe have observed occasional double ligation events on the 5\u2019 end of Read1, and we have found\nthat to fix this requires we run cutadapt twice.  Additionally, because two adapters are used for\neach library (to ensure proper balancing on the Illumina sequencer), two separate barcodes may\nbe ligated to the same Read1 5\u2019 end (often with 5\u2019 truncations).  To fix this we split the barcodes\nup into 15bp chunks so that cutadapt is able to deconvolute barcode adapters properly (as by\ndefault it will not find adapters missing the first N bases of the adapter sequence)\n\nparsebarcodes.sh writes the following files:\ntrimfirst_overlap_length.txt : file that always contains \"1\"\ntrimagain_overlap_length.txt : file that contains max((length of longest barcode - 2),5)\ng_adapters_default.fasta : empty file (to be fed to cutadapt properly)\na_adapters_default.fasta : empty file (to be fed to cutadapt properly)\ng_adapters.fasta :  fasta file containing sequences to be trimmed via cutadapt -g flag\na_adapters.fasta : fasta file containing sequences to be trimmed via cutadapt -a flag\nA_adapters.fasta : fasta file containing sequences to be trimmed via cutadapt -A flag\n\n  Usage: parsebarcodes.sh <randommer_length> <barcodes_fasta> <barcode_A> <barcode_B>\n",
            "id": "#parsebarcodes.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/eclip:0.6.0a_perl"
                }
            ],
            "baseCommand": [
                "compress_l2foldenrpeakfi_for_replicate_overlapping_bedformat.pl"
            ],
            "arguments": [
                "$(inputs.input_bed.nameroot).compressed.bed"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -1
                    },
                    "id": "#peakscompress.cwl/input_bed"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_bed.nameroot).compressed.bed"
                    },
                    "id": "#peakscompress.cwl/output_bed"
                }
            ],
            "doc": "This tool wraps compress_l2foldenrpeakfi_for_replicate_overlapping_bedformat.pl,\nwhich merges neighboring or overlapping regions in a BED file.\n  Usage:   perl compress_l2foldenrpeakfi_for_replicate_overlapping_bedformat.pl <in.bed> <out.compressed.bed>\n",
            "id": "#peakscompress.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "listing": [
                        {
                            "entryname": "$(inputs.newname + inputs.suffix)",
                            "entry": "$(inputs.srcfile)"
                        }
                    ],
                    "class": "InitialWorkDirRequirement"
                }
            ],
            "baseCommand": "true",
            "inputs": [
                {
                    "type": "string",
                    "id": "#rename.cwl/newname"
                },
                {
                    "type": "File",
                    "id": "#rename.cwl/srcfile"
                },
                {
                    "type": "string",
                    "id": "#rename.cwl/suffix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.newname + inputs.suffix)"
                    },
                    "id": "#rename.cwl/outfile"
                }
            ],
            "id": "#rename.cwl"
        },
        {
            "class": "CommandLineTool",
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/samtools:1.6"
                }
            ],
            "requirements": [
                {
                    "listing": [
                        "$(inputs.alignments)"
                    ],
                    "class": "InitialWorkDirRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "valueFrom": "$(self.basename)"
                    },
                    "label": "Input bam file.",
                    "id": "#samtools-index.cwl/alignments"
                }
            ],
            "baseCommand": [
                "samtools",
                "index",
                "-b"
            ],
            "outputs": [
                {
                    "type": "File",
                    "secondaryFiles": ".bai",
                    "outputBinding": {
                        "glob": "$(inputs.alignments.basename)"
                    },
                    "doc": "The index file",
                    "id": "#samtools-index.cwl/alignments_with_index"
                }
            ],
            "s:mainEntity": null,
            "s:downloadUrl": "https://github.com/common-workflow-language/workflows/blob/master/tools/samtools-index.cwl",
            "s:codeRepository": "https://github.com/common-workflow-language/workflows",
            "s:license": "http://www.apache.org/licenses/LICENSE-2.0",
            "s:isPartOf": {
                "class": "s:CreativeWork",
                "s:name": "Common Workflow Language",
                "s:url": "http://commonwl.org/"
            },
            "s:author": {
                "class": "s:Person",
                "s:name": "Andrey Kartashov",
                "s:email": "mailto:Andrey.Kartashov@cchmc.org",
                "s:sameAs": [
                    {
                        "id": "#0000-0001-9102-5681"
                    }
                ],
                "s:worksFor": [
                    {
                        "class": "s:Organization",
                        "s:name": "Cincinnati Children's Hospital Medical Center",
                        "s:location": "3333 Burnet Ave, Cincinnati, OH 45229-3026",
                        "s:department": [
                            {
                                "class": "s:Organization",
                                "s:name": "Barski Lab"
                            }
                        ]
                    }
                ]
            },
            "doc": "samtools-index.cwl is developed for CWL consortium\n",
            "id": "#samtools-index.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/samtools:1.6"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-L"
                    },
                    "doc": "only include reads overlapping this BED FILE [null]\n",
                    "id": "#samtools-mappedreadnum.cwl/bedoverlap"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-m"
                    },
                    "doc": "only include reads with number of CIGAR operations\nconsuming query sequence >= INT [0]\n",
                    "id": "#samtools-mappedreadnum.cwl/cigar"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "collapse the backward CIGAR operation\n",
                    "id": "#samtools-mappedreadnum.cwl/collapsecigar"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "doc": "print only the count of matching records\n",
                    "id": "#samtools-mappedreadnum.cwl/count"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-1"
                    },
                    "doc": "use fast BAM compression (implies -b)\n",
                    "id": "#samtools-mappedreadnum.cwl/fastcompression"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "doc": "Input bam file.\n",
                    "id": "#samtools-mappedreadnum.cwl/input"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-b"
                    },
                    "doc": "output in BAM format\n",
                    "id": "#samtools-mappedreadnum.cwl/isbam"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-C"
                    },
                    "doc": "output in CRAM format\n",
                    "id": "#samtools-mappedreadnum.cwl/iscram"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-o"
                    },
                    "id": "#samtools-mappedreadnum.cwl/output_name"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "integer part sets seed of random number generator [0];\nrest sets fraction of templates to subsample [no subsampling]\n",
                    "id": "#samtools-mappedreadnum.cwl/randomseed"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-r"
                    },
                    "doc": "only include reads in read group STR [null]\n",
                    "id": "#samtools-mappedreadnum.cwl/readsingroup"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-R"
                    },
                    "doc": "only include reads with read group listed in FILE [null]\n",
                    "id": "#samtools-mappedreadnum.cwl/readsingroupfile"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-l"
                    },
                    "doc": "only include reads in library STR [null]\n",
                    "id": "#samtools-mappedreadnum.cwl/readsinlibrary"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-q"
                    },
                    "doc": "only include reads with mapping quality >= INT [0]\n",
                    "id": "#samtools-mappedreadnum.cwl/readsquality"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "doc": "only include reads with all bits set in INT set in FLAG [0]\n",
                    "id": "#samtools-mappedreadnum.cwl/readswithbits"
                },
                {
                    "type": "int",
                    "default": 4,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-F"
                    },
                    "doc": "only include reads with none of the bits set in INT set in FLAG [0]\n",
                    "id": "#samtools-mappedreadnum.cwl/readswithoutbits"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "string"
                        }
                    ],
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "read tag to strip (repeatable) [null]\n",
                    "id": "#samtools-mappedreadnum.cwl/readtagtostrip"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-T"
                    },
                    "doc": "reference sequence FASTA FILE [null]\n",
                    "id": "#samtools-mappedreadnum.cwl/referencefasta"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "[region ...]\n",
                    "id": "#samtools-mappedreadnum.cwl/region"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-h"
                    },
                    "doc": "include header in SAM output\n",
                    "id": "#samtools-mappedreadnum.cwl/samheader"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-@"
                    },
                    "doc": "number of BAM compression threads [0]\n",
                    "id": "#samtools-mappedreadnum.cwl/threads"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-u"
                    },
                    "doc": "uncompressed BAM output (implies -b)\n",
                    "id": "#samtools-mappedreadnum.cwl/uncompressed"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_name)"
                    },
                    "id": "#samtools-mappedreadnum.cwl/output"
                }
            ],
            "baseCommand": [
                "samtools",
                "view"
            ],
            "s:downloadUrl": "https://github.com/common-workflow-language/workflows/blob/master/tools/samtools-view.cwl",
            "s:codeRepository": "https://github.com/common-workflow-language/workflows",
            "s:license": "http://www.apache.org/licenses/LICENSE-2.0",
            "s:isPartOf": {
                "class": "s:CreativeWork",
                "s:name": "Common Workflow Language",
                "s:url": "http://commonwl.org/"
            },
            "s:author": {
                "class": "s:Person",
                "s:name": "Andrey Kartashov",
                "s:email": "mailto:Andrey.Kartashov@cchmc.org",
                "s:sameAs": [
                    {
                        "$import": "#0000-0001-9102-5681"
                    }
                ],
                "s:worksFor": [
                    {
                        "class": "s:Organization",
                        "s:name": "Cincinnati Children's Hospital Medical Center",
                        "s:location": "3333 Burnet Ave, Cincinnati, OH 45229-3026",
                        "s:department": [
                            {
                                "class": "s:Organization",
                                "s:name": "Barski Lab"
                            }
                        ]
                    }
                ]
            },
            "doc": "samtools-view.cwl is developed for CWL consortium\n  Usage:   samtools view [options] <in.bam>|<in.sam>|<in.cram> [region ...]\n\n  Options: -b       output BAM\n           -C       output CRAM (requires -T)\n           -1       use fast BAM compression (implies -b)\n           -u       uncompressed BAM output (implies -b)\n           -h       include header in SAM output\n           -H       print SAM header only (no alignments)\n           -c       print only the count of matching records\n           -o FILE  output file name [stdout]\n           -U FILE  output reads not selected by filters to FILE [null]\n           -t FILE  FILE listing reference names and lengths (see long help) [null]\n           -T FILE  reference sequence FASTA FILE [null]\n           -L FILE  only include reads overlapping this BED FILE [null]\n           -r STR   only include reads in read group STR [null]\n           -R FILE  only include reads with read group listed in FILE [null]\n           -q INT   only include reads with mapping quality >= INT [0]\n           -l STR   only include reads in library STR [null]\n           -m INT   only include reads with number of CIGAR operations\n                    consuming query sequence >= INT [0]\n           -f INT   only include reads with all bits set in INT set in FLAG [0]\n           -F INT   only include reads with none of the bits set in INT\n                    set in FLAG [0]\n           -x STR   read tag to strip (repeatable) [null]\n           -B       collapse the backward CIGAR operation\n           -s FLOAT integer part sets seed of random number generator [0];\n                    rest sets fraction of templates to subsample [no subsampling]\n           -@ INT   number of BAM compression threads [0]\n",
            "id": "#samtools-mappedreadnum.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "baseCommand": [
                "samtools",
                "merge"
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/samtools:1.6"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 2
                    },
                    "label": "",
                    "doc": "input unmerged bam files",
                    "id": "#samtools-merge.cwl/input_bam_files"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 1,
                        "valueFrom": "${\n  if (inputs.output_bam == \"\") {\n    return inputs.input_bam_files[0].nameroot + \".merged.bam\";\n  }\n  else {\n    return inputs.output_bam;\n  }\n}\n"
                    },
                    "label": "",
                    "doc": "output merged bam file name",
                    "id": "#samtools-merge.cwl/output_bam"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_bam == \"\") {\n    return inputs.input_bam_files[0].nameroot + \".merged.bam\";\n  }\n  else {\n    return inputs.output_bam;\n  }\n}\n"
                    },
                    "label": "",
                    "doc": "output merged bam file",
                    "id": "#samtools-merge.cwl/output_bam_file"
                }
            ],
            "doc": "samtools-merge.cwl takes in a list of input_bam_files and\nreturns a merged BAM file.\n\nUsage: samtools merge [-nurlf] [-h inh.sam] [-b <bamlist.fofn>] <out.bam> <in1.bam> [<in2.bam> ... <inN.bam>]\n",
            "id": "#samtools-merge.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/samtools:1.6"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-L"
                    },
                    "doc": "only include reads overlapping this BED FILE [null]\n",
                    "id": "#samtools-viewr2.cwl/bedoverlap"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-m"
                    },
                    "doc": "only include reads with number of CIGAR operations\nconsuming query sequence >= INT [0]\n",
                    "id": "#samtools-viewr2.cwl/cigar"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "collapse the backward CIGAR operation\n",
                    "id": "#samtools-viewr2.cwl/collapsecigar"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "doc": "print only the count of matching records\n",
                    "id": "#samtools-viewr2.cwl/count"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-1"
                    },
                    "doc": "use fast BAM compression (implies -b)\n",
                    "id": "#samtools-viewr2.cwl/fastcompression"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "doc": "Input bam file.\n",
                    "id": "#samtools-viewr2.cwl/input"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-b"
                    },
                    "doc": "output in BAM format\n",
                    "id": "#samtools-viewr2.cwl/isbam"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-C"
                    },
                    "doc": "output in CRAM format\n",
                    "id": "#samtools-viewr2.cwl/iscram"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-o",
                        "valueFrom": "${\n  if (inputs.output_name == \"\") {\n    return inputs.input.nameroot + \".r2.bam\";\n  }\n  else {\n    return inputs.output_name;\n  }\n}\n"
                    },
                    "id": "#samtools-viewr2.cwl/output_name"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "integer part sets seed of random number generator [0];\nrest sets fraction of templates to subsample [no subsampling]\n",
                    "id": "#samtools-viewr2.cwl/randomseed"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-r"
                    },
                    "doc": "only include reads in read group STR [null]\n",
                    "id": "#samtools-viewr2.cwl/readsingroup"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-R"
                    },
                    "doc": "only include reads with read group listed in FILE [null]\n",
                    "id": "#samtools-viewr2.cwl/readsingroupfile"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-l"
                    },
                    "doc": "only include reads in library STR [null]\n",
                    "id": "#samtools-viewr2.cwl/readsinlibrary"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-q"
                    },
                    "doc": "only include reads with mapping quality >= INT [0]\n",
                    "id": "#samtools-viewr2.cwl/readsquality"
                },
                {
                    "type": "int",
                    "default": 128,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "doc": "only include reads with all bits set in INT set in FLAG [0]\n",
                    "id": "#samtools-viewr2.cwl/readswithbits"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-F"
                    },
                    "doc": "only include reads with none of the bits set in INT set in FLAG [0]\n",
                    "id": "#samtools-viewr2.cwl/readswithoutbits"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "string"
                        }
                    ],
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "read tag to strip (repeatable) [null]\n",
                    "id": "#samtools-viewr2.cwl/readtagtostrip"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-T"
                    },
                    "doc": "reference sequence FASTA FILE [null]\n",
                    "id": "#samtools-viewr2.cwl/referencefasta"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "[region ...]\n",
                    "id": "#samtools-viewr2.cwl/region"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-h"
                    },
                    "doc": "include header in SAM output\n",
                    "id": "#samtools-viewr2.cwl/samheader"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-@"
                    },
                    "doc": "number of BAM compression threads [0]\n",
                    "id": "#samtools-viewr2.cwl/threads"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-u"
                    },
                    "doc": "uncompressed BAM output (implies -b)\n",
                    "id": "#samtools-viewr2.cwl/uncompressed"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_name == \"\") {\n    return inputs.input.nameroot + \".r2.bam\";\n  }\n  else {\n    return inputs.output_name;\n  }\n}\n"
                    },
                    "id": "#samtools-viewr2.cwl/output"
                }
            ],
            "baseCommand": [
                "samtools",
                "view"
            ],
            "s:downloadUrl": "https://github.com/common-workflow-language/workflows/blob/master/tools/samtools-view.cwl",
            "s:codeRepository": "https://github.com/common-workflow-language/workflows",
            "s:license": "http://www.apache.org/licenses/LICENSE-2.0",
            "s:isPartOf": {
                "class": "s:CreativeWork",
                "s:name": "Common Workflow Language",
                "s:url": "http://commonwl.org/"
            },
            "s:author": {
                "class": "s:Person",
                "s:name": "Andrey Kartashov",
                "s:email": "mailto:Andrey.Kartashov@cchmc.org",
                "s:sameAs": [
                    {
                        "$import": "#0000-0001-9102-5681"
                    }
                ],
                "s:worksFor": [
                    {
                        "class": "s:Organization",
                        "s:name": "Cincinnati Children's Hospital Medical Center",
                        "s:location": "3333 Burnet Ave, Cincinnati, OH 45229-3026",
                        "s:department": [
                            {
                                "class": "s:Organization",
                                "s:name": "Barski Lab"
                            }
                        ]
                    }
                ]
            },
            "doc": "samtools-view.cwl is developed for CWL consortium\n  Usage:   samtools view [options] <in.bam>|<in.sam>|<in.cram> [region ...]\n\n  Options: -b       output BAM\n           -C       output CRAM (requires -T)\n           -1       use fast BAM compression (implies -b)\n           -u       uncompressed BAM output (implies -b)\n           -h       include header in SAM output\n           -H       print SAM header only (no alignments)\n           -c       print only the count of matching records\n           -o FILE  output file name [stdout]\n           -U FILE  output reads not selected by filters to FILE [null]\n           -t FILE  FILE listing reference names and lengths (see long help) [null]\n           -T FILE  reference sequence FASTA FILE [null]\n           -L FILE  only include reads overlapping this BED FILE [null]\n           -r STR   only include reads in read group STR [null]\n           -R FILE  only include reads with read group listed in FILE [null]\n           -q INT   only include reads with mapping quality >= INT [0]\n           -l STR   only include reads in library STR [null]\n           -m INT   only include reads with number of CIGAR operations\n                    consuming query sequence >= INT [0]\n           -f INT   only include reads with all bits set in INT set in FLAG [0]\n           -F INT   only include reads with none of the bits set in INT\n                    set in FLAG [0]\n           -x STR   read tag to strip (repeatable) [null]\n           -B       collapse the backward CIGAR operation\n           -s FLOAT integer part sets seed of random number generator [0];\n                    rest sets fraction of templates to subsample [no subsampling]\n           -@ INT   number of BAM compression threads [0]\n",
            "id": "#samtools-viewr2.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000,
                    "tmpdirMin": 8000,
                    "outdirMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/samtools:1.6"
                }
            ],
            "baseCommand": [
                "samtools",
                "sort"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "label": "",
                    "doc": "input bam",
                    "id": "#sort.cwl/input_sort_bam"
                },
                {
                    "default": "3G",
                    "type": "string",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-m"
                    },
                    "id": "#sort.cwl/memory"
                },
                {
                    "type": "boolean",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-n"
                    },
                    "default": false,
                    "id": "#sort.cwl/name_sort"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-o",
                        "valueFrom": "${\n  if (inputs.output_file == \"\") {\n    return inputs.input_sort_bam.nameroot + \"So.bam\";\n  }\n  else {\n    return inputs.output_file;\n  }\n}\n"
                    },
                    "default": "",
                    "id": "#sort.cwl/output_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_file == \"\") {\n    return inputs.input_sort_bam.nameroot + \"So.bam\";\n  }\n  else {\n    return inputs.output_file;\n  }\n}\n"
                    },
                    "label": "",
                    "doc": "sorted bam",
                    "id": "#sort.cwl/output_sort_bam"
                }
            ],
            "doc": "This tool wraps samtools sort by coordinates (namesort flag is False by default).\n  Usage: samtools sort [options...] [in.bam]\n",
            "id": "#sort.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 8,
                    "ramMin": 32000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/star:2.7.6a"
                }
            ],
            "inputs": [
                {
                    "default": "EndToEnd",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignEndsType"
                    },
                    "doc": "Local\nstring: type of read ends alignment\nLocal           ... standard local alignment with soft-clipping allowed\nEndToEnd        ... force end-to-end read alignment, do not soft-clip\nExtend5pOfRead1 ... fully extend only the 5p of the read1, all other ends: local alignment\n",
                    "id": "#star-genome.cwl/alignEndsType"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMax"
                    },
                    "doc": "0\nmaximum intron size, if 0, max intron size will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-genome.cwl/alignIntronMax"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMin"
                    },
                    "doc": "21\nminimum intron size: genomic gap is considered intron if its length>=alignIntronMin, otherwise it is considered Deletion\n",
                    "id": "#star-genome.cwl/alignIntronMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignMatesGapMax"
                    },
                    "doc": "0\nmaximum gap between two mates, if 0, max intron gap will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-genome.cwl/alignMatesGapMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJDBoverhangMin"
                    },
                    "doc": "3\nint>0: minimum overhang (i.e. block size) for annotated (sjdb) spliced alignments\n",
                    "id": "#star-genome.cwl/alignSJDBoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJoverhangMin"
                    },
                    "doc": "5\nint>0: minimum overhang (i.e. block size) for spliced alignments\n",
                    "id": "#star-genome.cwl/alignSJoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSoftClipAtReferenceEnds"
                    },
                    "doc": "Yes\nstring: allow the soft-clipping of the alignments past the end of the chromosomes\nYes ... allow\nNo  ... prohibit, useful for compatibility with Cufflinks\n",
                    "id": "#star-genome.cwl/alignSoftClipAtReferenceEnds"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLmin"
                    },
                    "doc": "0\nint>0: minimum mapped length for a read mate that is spliced\n",
                    "id": "#star-genome.cwl/alignSplicedMateMapLmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLminOverLmate"
                    },
                    "doc": "0.66\nfloat>0: alignSplicedMateMapLmin normalized to mate length\n",
                    "id": "#star-genome.cwl/alignSplicedMateMapLminOverLmate"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of different alignments per read to consider\n",
                    "id": "#star-genome.cwl/alignTranscriptsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerWindowNmax"
                    },
                    "doc": "100\nint>0: max number of transcripts per window\n",
                    "id": "#star-genome.cwl/alignTranscriptsPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignWindowsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of windows per read\n",
                    "id": "#star-genome.cwl/alignWindowsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesMate2basesN"
                    },
                    "doc": "int>0: number of bases from the 5' of mate 2 to use in collapsing (e.g. for\nRAMPAGE)\n",
                    "id": "#star-genome.cwl/bamRemoveDuplicatesMate2basesN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesType"
                    },
                    "doc": "-\nstring: mark duplicates in the BAM file, for now only works with sorted BAM feeded with inputBAMfile\n-               ... no duplicate removal/marking\nUniqueIdentical ... mark all multimappers, and duplicate unique mappers. The coordinates, FLAG, CIGAR must be identical\n",
                    "id": "#star-genome.cwl/bamRemoveDuplicatesType"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimJunctionOverhangMin"
                    },
                    "doc": "20\nint>=0: minimum overhang for a chimeric junction\n",
                    "id": "#star-genome.cwl/chimJunctionOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreDropMax"
                    },
                    "doc": "20\nint>=0: max drop (difference) of chimeric score (the sum of scores of all chimeric segements) from the read length\n",
                    "id": "#star-genome.cwl/chimScoreDropMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreJunctionNonGTAG"
                    },
                    "doc": "-1\nint: penalty for a non-GT/AG chimeric junction\n",
                    "id": "#star-genome.cwl/chimScoreJunctionNonGTAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreMin"
                    },
                    "doc": "0\nint>=0: minimum total (summed) score of the chimeric segments\n",
                    "id": "#star-genome.cwl/chimScoreMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreSeparation"
                    },
                    "doc": "int>=0: minimum difference (separation) between the best chimeric score and\nthe next one\n",
                    "id": "#star-genome.cwl/chimScoreSeparation"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimSegmentMin"
                    },
                    "doc": "0\nint>=0: minimum length of chimeric segment length, if ==0, no chimeric output\n",
                    "id": "#star-genome.cwl/chimSegmentMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterMMp"
                    },
                    "doc": "double(s): max proportion of mismatches for 3p adpater clipping for each\nmate.  If one value is given, it will be assumed the same for both mates.\n",
                    "id": "#star-genome.cwl/clip3pAdapterMMp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterSeq"
                    },
                    "doc": "string(s): adapter sequences to clip from 3p of each mate.  If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-genome.cwl/clip3pAdapterSeq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAfterAdapterNbases"
                    },
                    "doc": "int(s): number of bases to clip from 3p of each mate after the adapter\nclipping. If one value is given, it will be assumed the same for both\nmates.\n",
                    "id": "#star-genome.cwl/clip3pAfterAdapterNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 3p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-genome.cwl/clip3pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip5pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 5p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-genome.cwl/clip5pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeChrBinNbits"
                    },
                    "doc": "int: =log2(chrBin), where chrBin is the size of the bins for genome\nstorage: each chromosome will occupy an integer number of bins\n",
                    "id": "#star-genome.cwl/genomeChrBinNbits"
                },
                {
                    "type": [
                        "null",
                        "Directory"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeDir"
                    },
                    "doc": "string: path to the directory where genome files are stored (if\nrunMode!=generateGenome) or will be generated (if runMode==generateGenome)\n",
                    "id": "#star-genome.cwl/genomeDir"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "itemSeparator": " ",
                        "prefix": "--genomeFastaFiles"
                    },
                    "doc": "string(s): path(s) to the fasta files with genomic sequences for genome\ngeneration, separated by spaces. Only used if runMode==genomeGenerate.\nThese files should be plain text FASTA files, they *cannot* be zipped.\n",
                    "id": "#star-genome.cwl/genomeFastaFiles"
                },
                {
                    "default": "NoSharedMemory",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeLoad"
                    },
                    "doc": "NoSharedMemory\nstring: mode of shared memory usage for the genome files\nLoadAndKeep     ... load genome into shared and keep it in memory after run\nLoadAndRemove   ... load genome into shared but remove it after run\nLoadAndExit     ... load genome into shared memory and exit, keeping the genome in memory for future runs\nRemove          ... do not map anything, just remove loaded genome from memory\nNoSharedMemory  ... do not use shared memory, each job will have its own private copy of the genome\n",
                    "id": "#star-genome.cwl/genomeLoad"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAindexNbases"
                    },
                    "doc": "int: length (bases) of the SA pre-indexing string. Typically between 10 and\n15. Longer strings will use much more memory, but allow faster searches.\n",
                    "id": "#star-genome.cwl/genomeSAindexNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAsparseD"
                    },
                    "doc": "int>0: suffux array sparsity, i.e. distance between indices: use bigger\nnumbers to decrease needed RAM at the cost of mapping speed reduction\n",
                    "id": "#star-genome.cwl/genomeSAsparseD"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--inputBAMfile"
                    },
                    "doc": "string: path to BAM input file, to be used with --runMode\ninputAlignmentsFromBAM\n",
                    "id": "#star-genome.cwl/inputBAMfile"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitBAMsortRAM"
                    },
                    "doc": "int>=0: maximum available RAM for sorting BAM. If =0, it will be set to the\ngenome index size. 0 value can only be used with --genomeLoad\nNoSharedMemory option.\n",
                    "id": "#star-genome.cwl/limitBAMsortRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitGenomeGenerateRAM"
                    },
                    "doc": "31000000000\nint>0: maximum available RAM (bytes) for genome generation\n",
                    "id": "#star-genome.cwl/limitGenomeGenerateRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitIObufferSize"
                    },
                    "doc": "150000000\nint>0: max available buffers size (bytes) for input/output, per thread\n",
                    "id": "#star-genome.cwl/limitIObufferSize"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSAMoneReadBytes"
                    },
                    "doc": "100000\nint>0: max size of the SAM record for one read. Recommended value: >(2*(LengthMate1+LengthMate2+100)*outFilterMultimapNmax\n",
                    "id": "#star-genome.cwl/limitOutSAMoneReadBytes"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJcollapsed"
                    },
                    "doc": "1000000\nint>0: max number of collapsed junctions\n",
                    "id": "#star-genome.cwl/limitOutSJcollapsed"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJoneRead"
                    },
                    "doc": "1000\nint>0: max number of junctions for one read (including all multi-mappers)\n",
                    "id": "#star-genome.cwl/limitOutSJoneRead"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitSjdbInsertNsj"
                    },
                    "doc": "1000000\nint>=0: maximum number of junction to be inserted to the genome on the fly at the mapping stage, including those from annotations and those detected in the 1st step of the 2-pass run\n",
                    "id": "#star-genome.cwl/limitSjdbInsertNsj"
                },
                {
                    "type": "int",
                    "default": 10,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMcompression"
                    },
                    "doc": "int: -1 to 10  BAM compression level, -1=default compression (6?), 0=no\ncompression, 10=maximum compression\n",
                    "id": "#star-genome.cwl/outBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMsortingThreadN"
                    },
                    "doc": "int: >=0: number of threads for BAM sorting. 0 will default to\nmin(6,--runThreadN).\n",
                    "id": "#star-genome.cwl/outBAMsortingThreadN"
                },
                {
                    "default": "",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFileNamePrefix",
                        "valueFrom": "${\n  if (inputs.outFileNamePrefix == \"\") {\n    return inputs.readFilesIn[0].nameroot + \".STAR\";\n  }\n  else {\n    return inputs.outFileNamePrefix;\n  }\n}\n"
                    },
                    "doc": "string: output files name prefix (including full or relative path). Can\nonly be defined on the command line.\n",
                    "id": "#star-genome.cwl/outFileNamePrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterIntronMotifs"
                    },
                    "doc": "None\nstring: filter alignment using their motifs\nNone                           ... no filtering\nRemoveNoncanonical             ... filter out alignments that contain non-canonical junctions\nRemoveNoncanonicalUnannotated  ... filter out alignments that contain non-canonical unannotated junctions when using annotated splice junctions database. The annotated non-canonical junctions will be kept.\n",
                    "id": "#star-genome.cwl/outFilterIntronMotifs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNmin"
                    },
                    "doc": "0\nint: alignment will be output only if the number of matched bases is higher than this value\n",
                    "id": "#star-genome.cwl/outFilterMatchNmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNminOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterMatchNmin normalized to read length (sum of mates' lengths for paired-end reads)\n",
                    "id": "#star-genome.cwl/outFilterMatchNminOverLread"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNmax"
                    },
                    "doc": "10\nint: alignment will be output only if it has fewer mismatches than this value\n",
                    "id": "#star-genome.cwl/outFilterMismatchNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverLmax"
                    },
                    "doc": "0.3\nint: alignment will be output only if its ratio of mismatches to *mapped* length is less than this value\n",
                    "id": "#star-genome.cwl/outFilterMismatchNoverLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverReadLmax"
                    },
                    "doc": "1\nint: alignment will be output only if its ratio of mismatches to *read* length is less than this value\n",
                    "id": "#star-genome.cwl/outFilterMismatchNoverReadLmax"
                },
                {
                    "default": 1,
                    "type": "int",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapNmax"
                    },
                    "doc": "10\nint: read alignments will be output only if the read maps fewer than this value, otherwise no alignments will be output\n",
                    "id": "#star-genome.cwl/outFilterMultimapNmax"
                },
                {
                    "default": 1,
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapScoreRange"
                    },
                    "doc": "1\nint: the score range below the maximum score for multimapping alignments\n",
                    "id": "#star-genome.cwl/outFilterMultimapScoreRange"
                },
                {
                    "default": 10,
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMin"
                    },
                    "doc": "0\nint: alignment will be output only if its score is higher than this value\n",
                    "id": "#star-genome.cwl/outFilterScoreMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMinOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterScoreMin normalized to read length (sum of mates' lengths for paired-end reads)\n",
                    "id": "#star-genome.cwl/outFilterScoreMinOverLread"
                },
                {
                    "default": "BySJout",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterType"
                    },
                    "doc": "Normal\nstring: type of filtering\nNormal  ... standard filtering using only current alignment\nBySJout ... keep only those reads that contain junctions that passed filtering into SJ.out.tab\n",
                    "id": "#star-genome.cwl/outFilterType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outMultimapperOrder"
                    },
                    "doc": "Old_2.4\nstring: order of multimapping alignments in the output files\nOld_2.4             ... quasi-random order used before 2.5.0\nRandom              ... random order of alignments for each multi-mapper. Read mates (pairs) are always adjacent, all alignment for each read stay together. This option will become default in the future releases.\n",
                    "id": "#star-genome.cwl/outMultimapperOrder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outQSconversionAdd"
                    },
                    "doc": "int: add this number to the quality score (e.g. to convert from Illumina to Sanger, use -31)\n",
                    "id": "#star-genome.cwl/outQSconversionAdd"
                },
                {
                    "default": "Fastx",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outReadsUnmapped"
                    },
                    "doc": "None\nstring: output of unmapped reads (besides SAM)\nNone    ... no output\nFastx   ... output in separate fasta/fastq files, Unmapped.out.mate1/2\n",
                    "id": "#star-genome.cwl/outReadsUnmapped"
                },
                {
                    "default": "ID:foo",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattrRGline"
                    },
                    "doc": "-\nstring(s): SAM/BAM read group line. The first word contains the read group identifier and must start with \"ID:\", e.g. --outSAMattrRGline ID:xxx CN:yy \"DS:z z z\".\nxxx will be added as RG tag to each output alignment. Any spaces in the tag values have to be double quoted.\nComma separated RG lines correspons to different (comma separated) input files in --readFilesIn. Commas have to be surrounded by spaces, e.g.\n--outSAMattrRGline ID:xxx , ID:zzz \"DS:z z\" , ID:yyy DS:yyyy\n",
                    "id": "#star-genome.cwl/outSAMattrRGline"
                },
                {
                    "default": "All",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattributes"
                    },
                    "doc": "Standard\nstring: a string of desired SAM attributes, in the order desired for the output SAM\nNH HI AS nM NM MD jM jI XS ... any combination in any order\nStandard   ... NH HI AS nM\nAll        ... NH HI AS nM NM MD jM jI\nNone       ... no attributes\n",
                    "id": "#star-genome.cwl/outSAMattributes"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagAND"
                    },
                    "doc": "65535\nint: 0 to 65535: sam FLAG will be bitwise AND'd with this value, i.e. FLAG=FLAG & outSAMflagOR. This is applied after all flags have been set by STAR, but before outSAMflagOR. Can be used to unset specific bits that are not set otherwise.\n",
                    "id": "#star-genome.cwl/outSAMflagAND"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagOR"
                    },
                    "doc": "int: 0 to 65535: sam FLAG will be bitwise OR'd with this value, i.e.\nFLAG=FLAG | outSAMflagOR. This is applied after all flags have been set by\nSTAR, and after outSAMflagAND. Can be used to set specific bits that are\nnot set otherwise.\n",
                    "id": "#star-genome.cwl/outSAMflagOR"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderCommentFile"
                    },
                    "doc": "-\nstring: path to the file with @CO (comment) lines of the SAM header\n",
                    "id": "#star-genome.cwl/outSAMheaderCommentFile"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderHD"
                    },
                    "doc": "-\nstrings: @HD (header) line of the SAM header\n",
                    "id": "#star-genome.cwl/outSAMheaderHD"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderPG"
                    },
                    "doc": "-\nstrings: extra @PG (software) line of the SAM header (in addition to STAR)\n",
                    "id": "#star-genome.cwl/outSAMheaderPG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmapqUnique"
                    },
                    "doc": "255\nint: 0 to 255: the MAPQ value for unique mappers\n",
                    "id": "#star-genome.cwl/outSAMmapqUnique"
                },
                {
                    "type": "string",
                    "default": "Full",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmode"
                    },
                    "doc": "string: mode of SAM output\nNone ... no SAM output\nFull ... full SAM output\nNoQS ... full SAM but without quality scores\n",
                    "id": "#star-genome.cwl/outSAMmode"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMorder"
                    },
                    "doc": "Paired\nstring: type of sorting for the SAM output\nPaired: one mate after the other for all paired alignments\nPairedKeepInputOrder: one mate after the other for all paired alignments, the order is kept the same as in the input FASTQ files\n",
                    "id": "#star-genome.cwl/outSAMorder"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMprimaryFlag"
                    },
                    "doc": "OneBestScore\nstring: which alignments are considered primary - all others will be marked with 0x100 bit in the FLAG\nOneBestScore ... only one alignment with the best score is primary\nAllBestScore ... all alignments with the best score are primary\n",
                    "id": "#star-genome.cwl/outSAMprimaryFlag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMreadID"
                    },
                    "doc": "Standard\nstring: read ID record type\nStandard ... first word (until space) from the FASTx read ID line, removing /1,/2 from the end\nNumber   ... read number (index) in the FASTx file\n",
                    "id": "#star-genome.cwl/outSAMreadID"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMstrandField"
                    },
                    "doc": "None\nstring: Cufflinks-like strand field flag\nNone        ... not used\nintronMotif ... strand derived from the intron motif. Reads with inconsistent and/or non-canonical introns are filtered out.\n",
                    "id": "#star-genome.cwl/outSAMstrandField"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "default": [
                        "BAM",
                        "Unsorted"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMtype"
                    },
                    "doc": "strings: type of SAM/BAM output\n1st word:\nBAM  ... output BAM without sorting\nSAM  ... output SAM without sorting\nNone ... no SAM/BAM output\n2nd, 3rd:\nUnsorted           ... standard unsorted\nSortedByCoordinate ... sorted by coordinate. This option will allocate extra memory for sorting which can be specified by --limitBAMsortRAM.\n",
                    "id": "#star-genome.cwl/outSAMtype"
                },
                {
                    "default": "Within",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMunmapped"
                    },
                    "doc": "string: output of unmapped reads in the SAM format\nNone   ... no output\nWithin ... output unmapped reads within the main SAM file (i.e. Aligned.out.sam)\n",
                    "id": "#star-genome.cwl/outSAMunmapped"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountTotalMin"
                    },
                    "doc": "3   1   1   1 \n4 integers: minimum total (multi-mapping+unique) read count per junction for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\nJunctions are output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin conditions are satisfied\ndoes not apply to annotated junctions\n",
                    "id": "#star-genome.cwl/outSJfilterCountTotalMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountUniqueMin"
                    },
                    "doc": "3   1   1   1 \n4 integers: minimum uniquely mapping read count per junction for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\nJunctions are output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin conditions are satisfied\ndoes not apply to annotated junctions\n",
                    "id": "#star-genome.cwl/outSJfilterCountUniqueMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterDistToOtherSJmin"
                    },
                    "doc": "10  0   5   10\n4 integers>=0: minimum allowed distance to other junctions' donor/acceptor\ndoes not apply to annotated junctions\n",
                    "id": "#star-genome.cwl/outSJfilterDistToOtherSJmin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterIntronMaxVsReadN"
                    },
                    "doc": "50000 100000 200000\nN integers>=0: maximum gap allowed for junctions supported by 1,2,3,,,N reads\ni.e. by default junctions supported by 1 read can have gaps <=50000b, by 2 reads: <=100000b, by 3 reads: <=200000. by >=4 reads any gap <=alignIntronMax\ndoes not apply to annotated junctions\n",
                    "id": "#star-genome.cwl/outSJfilterIntronMaxVsReadN"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterOverhangMin"
                    },
                    "doc": "30  12  12  12\n4 integers:    minimum overhang length for splice junctions on both sides for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\ndoes not apply to annotated junctions\n",
                    "id": "#star-genome.cwl/outSJfilterOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterReads"
                    },
                    "doc": "All\nstring: which reads to consider for collapsed splice junctions output\nAll: all reads, unique- and multi-mappers\nUnique: uniquely mapping reads only\n",
                    "id": "#star-genome.cwl/outSJfilterReads"
                },
                {
                    "type": "string",
                    "default": "Log",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outStd"
                    },
                    "doc": "Log\nstring: which output will be directed to stdout (standard out)\nLog                    ... log messages\nSAM                    ... alignments in SAM format (which normally are output to Aligned.out.sam file), normal standard output will go into Log.std.out\nBAM_Unsorted           ... alignments in BAM format, unsorted. Requires --outSAMtype BAM Unsorted\nBAM_SortedByCoordinate ... alignments in BAM format, unsorted. Requires --outSAMtype BAM SortedByCoordinate\nBAM_Quant              ... alignments to transcriptome in BAM format, unsorted. Requires --quantMode TranscriptomeSAM\n",
                    "id": "#star-genome.cwl/outStd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outTmpDir"
                    },
                    "doc": "string: path to a directory that will be used as temporary by STAR. All contents of this directory will be removed!\n- the temp directory will default to outFileNamePrefix_STARtmp\n",
                    "id": "#star-genome.cwl/outTmpDir"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigNorm"
                    },
                    "doc": "RPM\nstring: type of normalization for the signal\nRPM    ... reads per million of mapped reads\nNone   ... no normalization, \"raw\" counts\n",
                    "id": "#star-genome.cwl/outWigNorm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigReferencesPrefix"
                    },
                    "doc": "string: prefix matching reference names to include in the output wiggle\nfile, e.g. \"chr\", default \"-\" - include all references\n",
                    "id": "#star-genome.cwl/outWigReferencesPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigStrand"
                    },
                    "doc": "Stranded\nstring: strandedness of wiggle/bedGraph output\nStranded   ...  separate strands, str1 and str2\nUnstranded ...  collapsed strands\n",
                    "id": "#star-genome.cwl/outWigStrand"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigType"
                    },
                    "doc": "None\nstring(s): type of signal output, e.g. \"bedGraph\" OR \"bedGraph read1_5p\". Requires sorted BAM: --outSAMtype BAM SortedByCoordinate .\n1st word:\nNone       ... no signal output\nbedGraph   ... bedGraph format\nwiggle     ... wiggle format\n2nd word:\nread1_5p   ... signal from only 5' of the 1st read, useful for CAGE/RAMPAGE etc\nread2      ... signal from only 2nd read\n",
                    "id": "#star-genome.cwl/outWigType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--parametersFiles"
                    },
                    "doc": "string: name of a user-defined parameters file, \"-\": none. Can only be\ndefined on the command line.\n",
                    "id": "#star-genome.cwl/parametersFiles"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantMode"
                    },
                    "doc": "-\nstring(s): types of quantification requested\n-                ... none\nTranscriptomeSAM ... output SAM/BAM alignments to transcriptome into a separate file\nGeneCounts       ... count reads per gene\n",
                    "id": "#star-genome.cwl/quantMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBAMcompression"
                    },
                    "doc": "int: -1 to 10  transcriptome BAM compression level, -1=default compression\n(6?), 0=no compression, 10=maximum compression\n",
                    "id": "#star-genome.cwl/quantTranscriptomeBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBan"
                    },
                    "doc": "IndelSoftclipSingleend\nstring: prohibit various alignment type\nIndelSoftclipSingleend  ... prohibit indels, soft clipping and single-end alignments - compatible with RSEM\nSingleend               ... prohibit single-end alignments\n",
                    "id": "#star-genome.cwl/quantTranscriptomeBan"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesCommand"
                    },
                    "doc": "string(s): command line to execute for each of the input file. This command should generate FASTA or FASTQ text and send it to stdout\nFor example: zcat - to uncompress .gz files, bzcat - to uncompress .bz2 files, etc.\n",
                    "id": "#star-genome.cwl/readFilesCommand"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesIn"
                    },
                    "doc": "string(s): paths to files that contain input read1 (and, if needed,  read2)\n",
                    "id": "#star-genome.cwl/readFilesIn"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMapNumber"
                    },
                    "doc": "-1\nint: number of reads to map from the beginning of the file\n-1: map all reads\n",
                    "id": "#star-genome.cwl/readMapNumber"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMatesLengthsIn"
                    },
                    "doc": "string: Equal/NotEqual - lengths of names,sequences,qualities for both\nmates are the same  / not the same. NotEqual is safe in all situations.\n",
                    "id": "#star-genome.cwl/readMatesLengthsIn"
                },
                {
                    "type": "string",
                    "default": "alignReads",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runMode"
                    },
                    "doc": "string: type of the run:\nalignReads             ... map reads\ngenomeGenerate         ... generate genome files\ninputAlignmentsFromBAM ... input alignments from BAM. Presently only works with --outWigType and --bamRemoveDuplicates.\n",
                    "id": "#star-genome.cwl/runMode"
                },
                {
                    "default": 8,
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runThreadN"
                    },
                    "doc": "1\nint: number of threads to run STAR\n",
                    "id": "#star-genome.cwl/runThreadN"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelBase"
                    },
                    "doc": "-2\ndeletion extension penalty per base (in addition to scoreDelOpen)\n",
                    "id": "#star-genome.cwl/scoreDelBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelOpen"
                    },
                    "doc": "-2\ndeletion open penalty\n",
                    "id": "#star-genome.cwl/scoreDelOpen"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGap"
                    },
                    "doc": "0\nint: splice junction penalty (independent on intron motif)\n",
                    "id": "#star-genome.cwl/scoreGap"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapATAC"
                    },
                    "doc": "-8\nAT/AC  and GT/AT junction penalty  (in addition to scoreGap)\n",
                    "id": "#star-genome.cwl/scoreGapATAC"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapGCAG"
                    },
                    "doc": "-4\nGC/AG and CT/GC junction penalty (in addition to scoreGap)\n",
                    "id": "#star-genome.cwl/scoreGapGCAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapNoncan"
                    },
                    "doc": "-8\nint: non-canonical junction penalty (in addition to scoreGap)\n",
                    "id": "#star-genome.cwl/scoreGapNoncan"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGenomicLengthLog2scale"
                    },
                    "doc": "-0.25\nextra score logarithmically scaled with genomic length of the alignment: scoreGenomicLengthLog2scale*log2(genomicLength)\n",
                    "id": "#star-genome.cwl/scoreGenomicLengthLog2scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsBase"
                    },
                    "doc": "-2\ninsertion extension penalty per base (in addition to scoreInsOpen)\n",
                    "id": "#star-genome.cwl/scoreInsBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsOpen"
                    },
                    "doc": "-2\ninsertion open penalty\n",
                    "id": "#star-genome.cwl/scoreInsOpen"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreStitchSJshift"
                    },
                    "doc": "1\nmaximum score reduction while searching for SJ boundaries inthe stitching step\n",
                    "id": "#star-genome.cwl/scoreStitchSJshift"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedMultimapNmax"
                    },
                    "doc": "10000\nint>0: only pieces that map fewer than this value are utilized in the stitching procedure\n",
                    "id": "#star-genome.cwl/seedMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedNoneLociPerWindow"
                    },
                    "doc": "10 \nint>0: max number of one seed loci per window\n",
                    "id": "#star-genome.cwl/seedNoneLociPerWindow"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerReadNmax"
                    },
                    "doc": "1000\nint>0: max number of seeds per read\n",
                    "id": "#star-genome.cwl/seedPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerWindowNmax"
                    },
                    "doc": "50\nint>0: max number of seeds per window\n",
                    "id": "#star-genome.cwl/seedPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchLmax"
                    },
                    "doc": "0\nint>=0: defines the maximum length of the seeds, if =0 max seed lengthis infinite\n",
                    "id": "#star-genome.cwl/seedSearchLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmax"
                    },
                    "doc": "50\nint>0: defines the search start point through the read - the read is split into pieces no longer than this value\n",
                    "id": "#star-genome.cwl/seedSearchStartLmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmaxOverLread"
                    },
                    "doc": "1.0\nfloat: seedSearchStartLmax normalized to read length (sum of mates' lengths for paired-end reads)\n",
                    "id": "#star-genome.cwl/seedSearchStartLmaxOverLread"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbFileChrStartEnd"
                    },
                    "doc": "-\nstring(s): path to the files with genomic coordinates (chr <tab> start <tab> end <tab> strand) for the splice junction introns. Multiple files can be supplied wand will be concatenated.\n",
                    "id": "#star-genome.cwl/sjdbFileChrStartEnd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFchrPrefix"
                    },
                    "doc": "string: prefix for chromosome names in a GTF file (e.g. 'chr' for using\nENSMEBL annotations with UCSC geneomes)\n",
                    "id": "#star-genome.cwl/sjdbGTFchrPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfeatureExon"
                    },
                    "doc": "exon\nstring: feature type in GTF file to be used as exons for building\ntranscripts\n",
                    "id": "#star-genome.cwl/sjdbGTFfeatureExon"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfile"
                    },
                    "doc": "string: path to the GTF file with annotations\n",
                    "id": "#star-genome.cwl/sjdbGTFfile"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentGene"
                    },
                    "doc": "gene_id\nstring: tag name to be used as exons' gene-parents (default \"gene_id\" works for GTF files)\n",
                    "id": "#star-genome.cwl/sjdbGTFtagExonParentGene"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentTranscript"
                    },
                    "doc": "transcript_id\nstring: tag name to be used as exons' transcript-parents (default\n\"transcript_id\" works for GTF files)\n",
                    "id": "#star-genome.cwl/sjdbGTFtagExonParentTranscript"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbOverhang"
                    },
                    "doc": "100\nint>0: length of the donor/acceptor sequence on each side of the junctions, ideally = (mate_length - 1)\n",
                    "id": "#star-genome.cwl/sjdbOverhang"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbScore"
                    },
                    "doc": "2\nint: extra alignment score for alignmets that cross database junctions\n",
                    "id": "#star-genome.cwl/sjdbScore"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sysShell"
                    },
                    "doc": "string: path to the shell binary, preferrably bash, e.g. /bin/bash.\n- ... the default shell is executed, typically /bin/sh. This was reported to fail on some Ubuntu systems - then you need to specify path to bash.\n",
                    "id": "#star-genome.cwl/sysShell"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopass1readsN"
                    },
                    "doc": "int: number of reads to process for the 1st step. Use very large number (or\ndefault -1) to map all reads in the first step.\n",
                    "id": "#star-genome.cwl/twopass1readsN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopassMode"
                    },
                    "doc": "None\nstring: 2-pass mapping mode.\nNone        ... 1-pass mapping\nBasic       ... basic 2-pass mapping, with all 1st pass junctions inserted into the genome indices on the fly\n",
                    "id": "#star-genome.cwl/twopassMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorDistNbins"
                    },
                    "doc": "9\nint>0: max number of bins between two anchors that allows aggregation of anchors into one window\n",
                    "id": "#star-genome.cwl/winAnchorDistNbins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorMultimapNmax"
                    },
                    "doc": "50\nint>0: max number of loci anchors are allowed to map to\n",
                    "id": "#star-genome.cwl/winAnchorMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winBinNbits"
                    },
                    "doc": "16\nint>0: =log2(winBin), where winBin is the size of the bin for the windows/clustering, each window will occupy an integer number of bins.\n",
                    "id": "#star-genome.cwl/winBinNbits"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winFlankNbins"
                    },
                    "doc": "4\nint>0: log2(winFlank), where win Flank is the size of the left and right flanking regions for each window\n",
                    "id": "#star-genome.cwl/winFlankNbins"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Aligned.out.bam"
                    },
                    "id": "#star-genome.cwl/aligned"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Log.final.out"
                    },
                    "id": "#star-genome.cwl/mappingstats"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate1"
                    },
                    "id": "#star-genome.cwl/output_map_unmapped_fwd"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate2"
                    },
                    "id": "#star-genome.cwl/output_map_unmapped_rev"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Log.out"
                    },
                    "id": "#star-genome.cwl/starsettings"
                }
            ],
            "baseCommand": [
                "STAR"
            ],
            "s:mainEntity": {
                "class": "s:SoftwareSourceCode",
                "s:name": "STAR",
                "s:about": "Aligns RNA-seq reads to a reference genome using uncompressed suffix arrays. STAR has a potential for accurately aligning long (several kilobases) reads that are emerging from the third-generation sequencing technologies.\n",
                "s:url": "https://github.com/alexdobin/STAR",
                "s:codeRepository": "https://github.com/alexdobin/STAR.git",
                "s:license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "s:targetProduct": {
                    "class": "s:SoftwareApplication",
                    "s:softwareVersion": "2.5.0b",
                    "s:applicationCategory": "commandline tool"
                },
                "s:programmingLanguage": "C++",
                "s:publication": [
                    {
                        "class": "s:ScholarlyArticle",
                        "id": "#bts635"
                    }
                ],
                "s:author": [
                    {
                        "class": "s:Person",
                        "id": "#dobin@cshl.edu",
                        "s:name": "Alexander Dobin",
                        "s:email": "#dobin@cshl.edu",
                        "s:worksFor": [
                            {
                                "class": "s:Organization",
                                "s:name": "Cold Spring Harbor Laboratory, Cold Spring Harbor, NY, USA"
                            }
                        ]
                    }
                ]
            },
            "s:downloadUrl": "https://github.com/common-workflow-language/workflows/blob/master/tools/STAR.cwl",
            "s:codeRepository": "https://github.com/common-workflow-language/workflows",
            "s:isPartOf": {
                "class": "s:CreativeWork",
                "s:name": "Common Workflow Language",
                "s:url": "http://commonwl.org/"
            },
            "s:author": {
                "class": "s:Person",
                "s:name": "Andrey Kartashov",
                "s:email": "mailto:Andrey.Kartashov@cchmc.org",
                "s:sameAs": [
                    {
                        "$import": "#0000-0001-9102-5681"
                    }
                ],
                "s:worksFor": [
                    {
                        "class": "s:Organization",
                        "s:name": "Cincinnati Children's Hospital Medical Center",
                        "s:location": "3333 Burnet Ave, Cincinnati, OH 45229-3026",
                        "s:department": [
                            {
                                "class": "s:Organization",
                                "s:name": "Barski Lab"
                            }
                        ]
                    }
                ]
            },
            "id": "#star-genome.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 8,
                    "ramMin": 32000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/star:2.7.6a"
                }
            ],
            "inputs": [
                {
                    "default": "EndToEnd",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignEndsType"
                    },
                    "doc": "Local\nstring: type of read ends alignment\nLocal           ... standard local alignment with soft-clipping allowed\nEndToEnd        ... force end-to-end read alignment, do not soft-clip\nExtend5pOfRead1 ... fully extend only the 5p of the read1, all other ends: local alignment\n",
                    "id": "#star-repeatmapping.cwl/alignEndsType"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMax"
                    },
                    "doc": "0\nmaximum intron size, if 0, max intron size will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-repeatmapping.cwl/alignIntronMax"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMin"
                    },
                    "doc": "21\nminimum intron size: genomic gap is considered intron if its length>=alignIntronMin, otherwise it is considered Deletion\n",
                    "id": "#star-repeatmapping.cwl/alignIntronMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignMatesGapMax"
                    },
                    "doc": "0\nmaximum gap between two mates, if 0, max intron gap will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-repeatmapping.cwl/alignMatesGapMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJDBoverhangMin"
                    },
                    "doc": "3\nint>0: minimum overhang (i.e. block size) for annotated (sjdb) spliced alignments\n",
                    "id": "#star-repeatmapping.cwl/alignSJDBoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJoverhangMin"
                    },
                    "doc": "5\nint>0: minimum overhang (i.e. block size) for spliced alignments\n",
                    "id": "#star-repeatmapping.cwl/alignSJoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSoftClipAtReferenceEnds"
                    },
                    "doc": "Yes\nstring: allow the soft-clipping of the alignments past the end of the chromosomes\nYes ... allow\nNo  ... prohibit, useful for compatibility with Cufflinks\n",
                    "id": "#star-repeatmapping.cwl/alignSoftClipAtReferenceEnds"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLmin"
                    },
                    "doc": "0\nint>0: minimum mapped length for a read mate that is spliced\n",
                    "id": "#star-repeatmapping.cwl/alignSplicedMateMapLmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLminOverLmate"
                    },
                    "doc": "0.66\nfloat>0: alignSplicedMateMapLmin normalized to mate length\n",
                    "id": "#star-repeatmapping.cwl/alignSplicedMateMapLminOverLmate"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of different alignments per read to consider\n",
                    "id": "#star-repeatmapping.cwl/alignTranscriptsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerWindowNmax"
                    },
                    "doc": "100\nint>0: max number of transcripts per window\n",
                    "id": "#star-repeatmapping.cwl/alignTranscriptsPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignWindowsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of windows per read\n",
                    "id": "#star-repeatmapping.cwl/alignWindowsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesMate2basesN"
                    },
                    "doc": "int>0: number of bases from the 5' of mate 2 to use in collapsing (e.g. for\nRAMPAGE)\n",
                    "id": "#star-repeatmapping.cwl/bamRemoveDuplicatesMate2basesN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesType"
                    },
                    "doc": "-\nstring: mark duplicates in the BAM file, for now only works with sorted BAM feeded with inputBAMfile\n-               ... no duplicate removal/marking\nUniqueIdentical ... mark all multimappers, and duplicate unique mappers. The coordinates, FLAG, CIGAR must be identical\n",
                    "id": "#star-repeatmapping.cwl/bamRemoveDuplicatesType"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimJunctionOverhangMin"
                    },
                    "doc": "20\nint>=0: minimum overhang for a chimeric junction\n",
                    "id": "#star-repeatmapping.cwl/chimJunctionOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreDropMax"
                    },
                    "doc": "20\nint>=0: max drop (difference) of chimeric score (the sum of scores of all chimeric segements) from the read length\n",
                    "id": "#star-repeatmapping.cwl/chimScoreDropMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreJunctionNonGTAG"
                    },
                    "doc": "-1\nint: penalty for a non-GT/AG chimeric junction\n",
                    "id": "#star-repeatmapping.cwl/chimScoreJunctionNonGTAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreMin"
                    },
                    "doc": "0\nint>=0: minimum total (summed) score of the chimeric segments\n",
                    "id": "#star-repeatmapping.cwl/chimScoreMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreSeparation"
                    },
                    "doc": "int>=0: minimum difference (separation) between the best chimeric score and\nthe next one\n",
                    "id": "#star-repeatmapping.cwl/chimScoreSeparation"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimSegmentMin"
                    },
                    "doc": "0\nint>=0: minimum length of chimeric segment length, if ==0, no chimeric output\n",
                    "id": "#star-repeatmapping.cwl/chimSegmentMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterMMp"
                    },
                    "doc": "double(s): max proportion of mismatches for 3p adpater clipping for each\nmate.  If one value is given, it will be assumed the same for both mates.\n",
                    "id": "#star-repeatmapping.cwl/clip3pAdapterMMp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterSeq"
                    },
                    "doc": "string(s): adapter sequences to clip from 3p of each mate.  If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-repeatmapping.cwl/clip3pAdapterSeq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAfterAdapterNbases"
                    },
                    "doc": "int(s): number of bases to clip from 3p of each mate after the adapter\nclipping. If one value is given, it will be assumed the same for both\nmates.\n",
                    "id": "#star-repeatmapping.cwl/clip3pAfterAdapterNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 3p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-repeatmapping.cwl/clip3pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip5pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 5p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-repeatmapping.cwl/clip5pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeChrBinNbits"
                    },
                    "doc": "int: =log2(chrBin), where chrBin is the size of the bins for genome\nstorage: each chromosome will occupy an integer number of bins\n",
                    "id": "#star-repeatmapping.cwl/genomeChrBinNbits"
                },
                {
                    "type": [
                        "null",
                        "Directory"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeDir"
                    },
                    "doc": "string: path to the directory where genome files are stored (if\nrunMode!=generateGenome) or will be generated (if runMode==generateGenome)\n",
                    "id": "#star-repeatmapping.cwl/genomeDir"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "itemSeparator": " ",
                        "prefix": "--genomeFastaFiles"
                    },
                    "doc": "string(s): path(s) to the fasta files with genomic sequences for genome\ngeneration, separated by spaces. Only used if runMode==genomeGenerate.\nThese files should be plain text FASTA files, they *cannot* be zipped.\n",
                    "id": "#star-repeatmapping.cwl/genomeFastaFiles"
                },
                {
                    "default": "NoSharedMemory",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeLoad"
                    },
                    "doc": "NoSharedMemory\nstring: mode of shared memory usage for the genome files\nLoadAndKeep     ... load genome into shared and keep it in memory after run\nLoadAndRemove   ... load genome into shared but remove it after run\nLoadAndExit     ... load genome into shared memory and exit, keeping the genome in memory for future runs\nRemove          ... do not map anything, just remove loaded genome from memory\nNoSharedMemory  ... do not use shared memory, each job will have its own private copy of the genome\n",
                    "id": "#star-repeatmapping.cwl/genomeLoad"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAindexNbases"
                    },
                    "doc": "int: length (bases) of the SA pre-indexing string. Typically between 10 and\n15. Longer strings will use much more memory, but allow faster searches.\n",
                    "id": "#star-repeatmapping.cwl/genomeSAindexNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAsparseD"
                    },
                    "doc": "int>0: suffux array sparsity, i.e. distance between indices: use bigger\nnumbers to decrease needed RAM at the cost of mapping speed reduction\n",
                    "id": "#star-repeatmapping.cwl/genomeSAsparseD"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--inputBAMfile"
                    },
                    "doc": "string: path to BAM input file, to be used with --runMode\ninputAlignmentsFromBAM\n",
                    "id": "#star-repeatmapping.cwl/inputBAMfile"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitBAMsortRAM"
                    },
                    "doc": "int>=0: maximum available RAM for sorting BAM. If =0, it will be set to the\ngenome index size. 0 value can only be used with --genomeLoad\nNoSharedMemory option.\n",
                    "id": "#star-repeatmapping.cwl/limitBAMsortRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitGenomeGenerateRAM"
                    },
                    "doc": "31000000000\nint>0: maximum available RAM (bytes) for genome generation\n",
                    "id": "#star-repeatmapping.cwl/limitGenomeGenerateRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitIObufferSize"
                    },
                    "doc": "150000000\nint>0: max available buffers size (bytes) for input/output, per thread\n",
                    "id": "#star-repeatmapping.cwl/limitIObufferSize"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSAMoneReadBytes"
                    },
                    "doc": "100000\nint>0: max size of the SAM record for one read. Recommended value: >(2*(LengthMate1+LengthMate2+100)*outFilterMultimapNmax\n",
                    "id": "#star-repeatmapping.cwl/limitOutSAMoneReadBytes"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJcollapsed"
                    },
                    "doc": "1000000\nint>0: max number of collapsed junctions\n",
                    "id": "#star-repeatmapping.cwl/limitOutSJcollapsed"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJoneRead"
                    },
                    "doc": "1000\nint>0: max number of junctions for one read (including all multi-mappers)\n",
                    "id": "#star-repeatmapping.cwl/limitOutSJoneRead"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitSjdbInsertNsj"
                    },
                    "doc": "1000000\nint>=0: maximum number of junction to be inserted to the genome on the fly at the mapping stage, including those from annotations and those detected in the 1st step of the 2-pass run\n",
                    "id": "#star-repeatmapping.cwl/limitSjdbInsertNsj"
                },
                {
                    "type": "int",
                    "default": 10,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMcompression"
                    },
                    "doc": "int: -1 to 10  BAM compression level, -1=default compression (6?), 0=no\ncompression, 10=maximum compression\n",
                    "id": "#star-repeatmapping.cwl/outBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMsortingThreadN"
                    },
                    "doc": "int: >=0: number of threads for BAM sorting. 0 will default to\nmin(6,--runThreadN).\n",
                    "id": "#star-repeatmapping.cwl/outBAMsortingThreadN"
                },
                {
                    "default": "",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFileNamePrefix",
                        "valueFrom": "${\n  if (inputs.outFileNamePrefix == \"\") {\n    return inputs.readFilesIn[0].nameroot + \".STAR\";\n  }\n  else {\n    return inputs.outFileNamePrefix;\n  }\n}\n"
                    },
                    "doc": "string: output files name prefix (including full or relative path). Can\nonly be defined on the command line.\n",
                    "id": "#star-repeatmapping.cwl/outFileNamePrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterIntronMotifs"
                    },
                    "doc": "None\nstring: filter alignment using their motifs\nNone                           ... no filtering\nRemoveNoncanonical             ... filter out alignments that contain non-canonical junctions\nRemoveNoncanonicalUnannotated  ... filter out alignments that contain non-canonical unannotated junctions when using annotated splice junctions database. The annotated non-canonical junctions will be kept.\n",
                    "id": "#star-repeatmapping.cwl/outFilterIntronMotifs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNmin"
                    },
                    "doc": "0\nint: alignment will be output only if the number of matched bases is higher than this value\n",
                    "id": "#star-repeatmapping.cwl/outFilterMatchNmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNminOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterMatchNmin normalized to read length (sum of mates' lengths for paired-end reads)\n",
                    "id": "#star-repeatmapping.cwl/outFilterMatchNminOverLread"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNmax"
                    },
                    "doc": "10\nint: alignment will be output only if it has fewer mismatches than this value\n",
                    "id": "#star-repeatmapping.cwl/outFilterMismatchNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverLmax"
                    },
                    "doc": "0.3\nint: alignment will be output only if its ratio of mismatches to *mapped* length is less than this value\n",
                    "id": "#star-repeatmapping.cwl/outFilterMismatchNoverLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverReadLmax"
                    },
                    "doc": "1\nint: alignment will be output only if its ratio of mismatches to *read* length is less than this value\n",
                    "id": "#star-repeatmapping.cwl/outFilterMismatchNoverReadLmax"
                },
                {
                    "default": 30,
                    "type": "int",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapNmax"
                    },
                    "doc": "10\nint: read alignments will be output only if the read maps fewer than this value, otherwise no alignments will be output\n",
                    "id": "#star-repeatmapping.cwl/outFilterMultimapNmax"
                },
                {
                    "default": 1,
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapScoreRange"
                    },
                    "doc": "1\nint: the score range below the maximum score for multimapping alignments\n",
                    "id": "#star-repeatmapping.cwl/outFilterMultimapScoreRange"
                },
                {
                    "default": 10,
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMin"
                    },
                    "doc": "0\nint: alignment will be output only if its score is higher than this value\n",
                    "id": "#star-repeatmapping.cwl/outFilterScoreMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMinOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterScoreMin normalized to read length (sum of mates' lengths for paired-end reads)\n",
                    "id": "#star-repeatmapping.cwl/outFilterScoreMinOverLread"
                },
                {
                    "default": "BySJout",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterType"
                    },
                    "doc": "Normal\nstring: type of filtering\nNormal  ... standard filtering using only current alignment\nBySJout ... keep only those reads that contain junctions that passed filtering into SJ.out.tab\n",
                    "id": "#star-repeatmapping.cwl/outFilterType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outMultimapperOrder"
                    },
                    "doc": "Old_2.4\nstring: order of multimapping alignments in the output files\nOld_2.4             ... quasi-random order used before 2.5.0\nRandom              ... random order of alignments for each multi-mapper. Read mates (pairs) are always adjacent, all alignment for each read stay together. This option will become default in the future releases.\n",
                    "id": "#star-repeatmapping.cwl/outMultimapperOrder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outQSconversionAdd"
                    },
                    "doc": "int: add this number to the quality score (e.g. to convert from Illumina to Sanger, use -31)\n",
                    "id": "#star-repeatmapping.cwl/outQSconversionAdd"
                },
                {
                    "default": "Fastx",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outReadsUnmapped"
                    },
                    "doc": "None\nstring: output of unmapped reads (besides SAM)\nNone    ... no output\nFastx   ... output in separate fasta/fastq files, Unmapped.out.mate1/2\n",
                    "id": "#star-repeatmapping.cwl/outReadsUnmapped"
                },
                {
                    "default": "ID:foo",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattrRGline"
                    },
                    "doc": "-\nstring(s): SAM/BAM read group line. The first word contains the read group identifier and must start with \"ID:\", e.g. --outSAMattrRGline ID:xxx CN:yy \"DS:z z z\".\nxxx will be added as RG tag to each output alignment. Any spaces in the tag values have to be double quoted.\nComma separated RG lines correspons to different (comma separated) input files in --readFilesIn. Commas have to be surrounded by spaces, e.g.\n--outSAMattrRGline ID:xxx , ID:zzz \"DS:z z\" , ID:yyy DS:yyyy\n",
                    "id": "#star-repeatmapping.cwl/outSAMattrRGline"
                },
                {
                    "default": "All",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattributes"
                    },
                    "doc": "Standard\nstring: a string of desired SAM attributes, in the order desired for the output SAM\nNH HI AS nM NM MD jM jI XS ... any combination in any order\nStandard   ... NH HI AS nM\nAll        ... NH HI AS nM NM MD jM jI\nNone       ... no attributes\n",
                    "id": "#star-repeatmapping.cwl/outSAMattributes"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagAND"
                    },
                    "doc": "65535\nint: 0 to 65535: sam FLAG will be bitwise AND'd with this value, i.e. FLAG=FLAG & outSAMflagOR. This is applied after all flags have been set by STAR, but before outSAMflagOR. Can be used to unset specific bits that are not set otherwise.\n",
                    "id": "#star-repeatmapping.cwl/outSAMflagAND"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagOR"
                    },
                    "doc": "int: 0 to 65535: sam FLAG will be bitwise OR'd with this value, i.e.\nFLAG=FLAG | outSAMflagOR. This is applied after all flags have been set by\nSTAR, and after outSAMflagAND. Can be used to set specific bits that are\nnot set otherwise.\n",
                    "id": "#star-repeatmapping.cwl/outSAMflagOR"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderCommentFile"
                    },
                    "doc": "-\nstring: path to the file with @CO (comment) lines of the SAM header\n",
                    "id": "#star-repeatmapping.cwl/outSAMheaderCommentFile"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderHD"
                    },
                    "doc": "-\nstrings: @HD (header) line of the SAM header\n",
                    "id": "#star-repeatmapping.cwl/outSAMheaderHD"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderPG"
                    },
                    "doc": "-\nstrings: extra @PG (software) line of the SAM header (in addition to STAR)\n",
                    "id": "#star-repeatmapping.cwl/outSAMheaderPG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmapqUnique"
                    },
                    "doc": "255\nint: 0 to 255: the MAPQ value for unique mappers\n",
                    "id": "#star-repeatmapping.cwl/outSAMmapqUnique"
                },
                {
                    "type": "string",
                    "default": "Full",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmode"
                    },
                    "doc": "string: mode of SAM output\nNone ... no SAM output\nFull ... full SAM output\nNoQS ... full SAM but without quality scores\n",
                    "id": "#star-repeatmapping.cwl/outSAMmode"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMorder"
                    },
                    "doc": "Paired\nstring: type of sorting for the SAM output\nPaired: one mate after the other for all paired alignments\nPairedKeepInputOrder: one mate after the other for all paired alignments, the order is kept the same as in the input FASTQ files\n",
                    "id": "#star-repeatmapping.cwl/outSAMorder"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMprimaryFlag"
                    },
                    "doc": "OneBestScore\nstring: which alignments are considered primary - all others will be marked with 0x100 bit in the FLAG\nOneBestScore ... only one alignment with the best score is primary\nAllBestScore ... all alignments with the best score are primary\n",
                    "id": "#star-repeatmapping.cwl/outSAMprimaryFlag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMreadID"
                    },
                    "doc": "Standard\nstring: read ID record type\nStandard ... first word (until space) from the FASTx read ID line, removing /1,/2 from the end\nNumber   ... read number (index) in the FASTx file\n",
                    "id": "#star-repeatmapping.cwl/outSAMreadID"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMstrandField"
                    },
                    "doc": "None\nstring: Cufflinks-like strand field flag\nNone        ... not used\nintronMotif ... strand derived from the intron motif. Reads with inconsistent and/or non-canonical introns are filtered out.\n",
                    "id": "#star-repeatmapping.cwl/outSAMstrandField"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "default": [
                        "BAM",
                        "Unsorted"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMtype"
                    },
                    "doc": "strings: type of SAM/BAM output\n1st word:\nBAM  ... output BAM without sorting\nSAM  ... output SAM without sorting\nNone ... no SAM/BAM output\n2nd, 3rd:\nUnsorted           ... standard unsorted\nSortedByCoordinate ... sorted by coordinate. This option will allocate extra memory for sorting which can be specified by --limitBAMsortRAM.\n",
                    "id": "#star-repeatmapping.cwl/outSAMtype"
                },
                {
                    "default": "Within",
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMunmapped"
                    },
                    "doc": "string: output of unmapped reads in the SAM format\nNone   ... no output\nWithin ... output unmapped reads within the main SAM file (i.e. Aligned.out.sam)\n",
                    "id": "#star-repeatmapping.cwl/outSAMunmapped"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountTotalMin"
                    },
                    "doc": "3   1   1   1 \n4 integers: minimum total (multi-mapping+unique) read count per junction for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\nJunctions are output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin conditions are satisfied\ndoes not apply to annotated junctions\n",
                    "id": "#star-repeatmapping.cwl/outSJfilterCountTotalMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountUniqueMin"
                    },
                    "doc": "3   1   1   1 \n4 integers: minimum uniquely mapping read count per junction for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\nJunctions are output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin conditions are satisfied\ndoes not apply to annotated junctions\n",
                    "id": "#star-repeatmapping.cwl/outSJfilterCountUniqueMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterDistToOtherSJmin"
                    },
                    "doc": "10  0   5   10\n4 integers>=0: minimum allowed distance to other junctions' donor/acceptor\ndoes not apply to annotated junctions\n",
                    "id": "#star-repeatmapping.cwl/outSJfilterDistToOtherSJmin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterIntronMaxVsReadN"
                    },
                    "doc": "50000 100000 200000\nN integers>=0: maximum gap allowed for junctions supported by 1,2,3,,,N reads\ni.e. by default junctions supported by 1 read can have gaps <=50000b, by 2 reads: <=100000b, by 3 reads: <=200000. by >=4 reads any gap <=alignIntronMax\ndoes not apply to annotated junctions\n",
                    "id": "#star-repeatmapping.cwl/outSJfilterIntronMaxVsReadN"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterOverhangMin"
                    },
                    "doc": "30  12  12  12\n4 integers:    minimum overhang length for splice junctions on both sides for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\ndoes not apply to annotated junctions\n",
                    "id": "#star-repeatmapping.cwl/outSJfilterOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterReads"
                    },
                    "doc": "All\nstring: which reads to consider for collapsed splice junctions output\nAll: all reads, unique- and multi-mappers\nUnique: uniquely mapping reads only\n",
                    "id": "#star-repeatmapping.cwl/outSJfilterReads"
                },
                {
                    "type": "string",
                    "default": "Log",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outStd"
                    },
                    "doc": "Log\nstring: which output will be directed to stdout (standard out)\nLog                    ... log messages\nSAM                    ... alignments in SAM format (which normally are output to Aligned.out.sam file), normal standard output will go into Log.std.out\nBAM_Unsorted           ... alignments in BAM format, unsorted. Requires --outSAMtype BAM Unsorted\nBAM_SortedByCoordinate ... alignments in BAM format, unsorted. Requires --outSAMtype BAM SortedByCoordinate\nBAM_Quant              ... alignments to transcriptome in BAM format, unsorted. Requires --quantMode TranscriptomeSAM\n",
                    "id": "#star-repeatmapping.cwl/outStd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outTmpDir"
                    },
                    "doc": "string: path to a directory that will be used as temporary by STAR. All contents of this directory will be removed!\n- the temp directory will default to outFileNamePrefix_STARtmp\n",
                    "id": "#star-repeatmapping.cwl/outTmpDir"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigNorm"
                    },
                    "doc": "RPM\nstring: type of normalization for the signal\nRPM    ... reads per million of mapped reads\nNone   ... no normalization, \"raw\" counts\n",
                    "id": "#star-repeatmapping.cwl/outWigNorm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigReferencesPrefix"
                    },
                    "doc": "string: prefix matching reference names to include in the output wiggle\nfile, e.g. \"chr\", default \"-\" - include all references\n",
                    "id": "#star-repeatmapping.cwl/outWigReferencesPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigStrand"
                    },
                    "doc": "Stranded\nstring: strandedness of wiggle/bedGraph output\nStranded   ...  separate strands, str1 and str2\nUnstranded ...  collapsed strands\n",
                    "id": "#star-repeatmapping.cwl/outWigStrand"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigType"
                    },
                    "doc": "None\nstring(s): type of signal output, e.g. \"bedGraph\" OR \"bedGraph read1_5p\". Requires sorted BAM: --outSAMtype BAM SortedByCoordinate .\n1st word:\nNone       ... no signal output\nbedGraph   ... bedGraph format\nwiggle     ... wiggle format\n2nd word:\nread1_5p   ... signal from only 5' of the 1st read, useful for CAGE/RAMPAGE etc\nread2      ... signal from only 2nd read\n",
                    "id": "#star-repeatmapping.cwl/outWigType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--parametersFiles"
                    },
                    "doc": "string: name of a user-defined parameters file, \"-\": none. Can only be\ndefined on the command line.\n",
                    "id": "#star-repeatmapping.cwl/parametersFiles"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantMode"
                    },
                    "doc": "-\nstring(s): types of quantification requested\n-                ... none\nTranscriptomeSAM ... output SAM/BAM alignments to transcriptome into a separate file\nGeneCounts       ... count reads per gene\n",
                    "id": "#star-repeatmapping.cwl/quantMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBAMcompression"
                    },
                    "doc": "int: -1 to 10  transcriptome BAM compression level, -1=default compression\n(6?), 0=no compression, 10=maximum compression\n",
                    "id": "#star-repeatmapping.cwl/quantTranscriptomeBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBan"
                    },
                    "doc": "IndelSoftclipSingleend\nstring: prohibit various alignment type\nIndelSoftclipSingleend  ... prohibit indels, soft clipping and single-end alignments - compatible with RSEM\nSingleend               ... prohibit single-end alignments\n",
                    "id": "#star-repeatmapping.cwl/quantTranscriptomeBan"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesCommand"
                    },
                    "doc": "string(s): command line to execute for each of the input file. This command should generate FASTA or FASTQ text and send it to stdout\nFor example: zcat - to uncompress .gz files, bzcat - to uncompress .bz2 files, etc.\n",
                    "id": "#star-repeatmapping.cwl/readFilesCommand"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesIn"
                    },
                    "doc": "string(s): paths to files that contain input read1 (and, if needed,  read2)\n",
                    "id": "#star-repeatmapping.cwl/readFilesIn"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMapNumber"
                    },
                    "doc": "-1\nint: number of reads to map from the beginning of the file\n-1: map all reads\n",
                    "id": "#star-repeatmapping.cwl/readMapNumber"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMatesLengthsIn"
                    },
                    "doc": "string: Equal/NotEqual - lengths of names,sequences,qualities for both\nmates are the same  / not the same. NotEqual is safe in all situations.\n",
                    "id": "#star-repeatmapping.cwl/readMatesLengthsIn"
                },
                {
                    "type": "string",
                    "default": "alignReads",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runMode"
                    },
                    "doc": "string: type of the run:\nalignReads             ... map reads\ngenomeGenerate         ... generate genome files\ninputAlignmentsFromBAM ... input alignments from BAM. Presently only works with --outWigType and --bamRemoveDuplicates.\n",
                    "id": "#star-repeatmapping.cwl/runMode"
                },
                {
                    "default": 8,
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runThreadN"
                    },
                    "doc": "1\nint: number of threads to run STAR\n",
                    "id": "#star-repeatmapping.cwl/runThreadN"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelBase"
                    },
                    "doc": "-2\ndeletion extension penalty per base (in addition to scoreDelOpen)\n",
                    "id": "#star-repeatmapping.cwl/scoreDelBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelOpen"
                    },
                    "doc": "-2\ndeletion open penalty\n",
                    "id": "#star-repeatmapping.cwl/scoreDelOpen"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGap"
                    },
                    "doc": "0\nint: splice junction penalty (independent on intron motif)\n",
                    "id": "#star-repeatmapping.cwl/scoreGap"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapATAC"
                    },
                    "doc": "-8\nAT/AC  and GT/AT junction penalty  (in addition to scoreGap)\n",
                    "id": "#star-repeatmapping.cwl/scoreGapATAC"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapGCAG"
                    },
                    "doc": "-4\nGC/AG and CT/GC junction penalty (in addition to scoreGap)\n",
                    "id": "#star-repeatmapping.cwl/scoreGapGCAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapNoncan"
                    },
                    "doc": "-8\nint: non-canonical junction penalty (in addition to scoreGap)\n",
                    "id": "#star-repeatmapping.cwl/scoreGapNoncan"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGenomicLengthLog2scale"
                    },
                    "doc": "-0.25\nextra score logarithmically scaled with genomic length of the alignment: scoreGenomicLengthLog2scale*log2(genomicLength)\n",
                    "id": "#star-repeatmapping.cwl/scoreGenomicLengthLog2scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsBase"
                    },
                    "doc": "-2\ninsertion extension penalty per base (in addition to scoreInsOpen)\n",
                    "id": "#star-repeatmapping.cwl/scoreInsBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsOpen"
                    },
                    "doc": "-2\ninsertion open penalty\n",
                    "id": "#star-repeatmapping.cwl/scoreInsOpen"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreStitchSJshift"
                    },
                    "doc": "1\nmaximum score reduction while searching for SJ boundaries inthe stitching step\n",
                    "id": "#star-repeatmapping.cwl/scoreStitchSJshift"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedMultimapNmax"
                    },
                    "doc": "10000\nint>0: only pieces that map fewer than this value are utilized in the stitching procedure\n",
                    "id": "#star-repeatmapping.cwl/seedMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedNoneLociPerWindow"
                    },
                    "doc": "10 \nint>0: max number of one seed loci per window\n",
                    "id": "#star-repeatmapping.cwl/seedNoneLociPerWindow"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerReadNmax"
                    },
                    "doc": "1000\nint>0: max number of seeds per read\n",
                    "id": "#star-repeatmapping.cwl/seedPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerWindowNmax"
                    },
                    "doc": "50\nint>0: max number of seeds per window\n",
                    "id": "#star-repeatmapping.cwl/seedPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchLmax"
                    },
                    "doc": "0\nint>=0: defines the maximum length of the seeds, if =0 max seed lengthis infinite\n",
                    "id": "#star-repeatmapping.cwl/seedSearchLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmax"
                    },
                    "doc": "50\nint>0: defines the search start point through the read - the read is split into pieces no longer than this value\n",
                    "id": "#star-repeatmapping.cwl/seedSearchStartLmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmaxOverLread"
                    },
                    "doc": "1.0\nfloat: seedSearchStartLmax normalized to read length (sum of mates' lengths for paired-end reads)\n",
                    "id": "#star-repeatmapping.cwl/seedSearchStartLmaxOverLread"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbFileChrStartEnd"
                    },
                    "doc": "-\nstring(s): path to the files with genomic coordinates (chr <tab> start <tab> end <tab> strand) for the splice junction introns. Multiple files can be supplied wand will be concatenated.\n",
                    "id": "#star-repeatmapping.cwl/sjdbFileChrStartEnd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFchrPrefix"
                    },
                    "doc": "string: prefix for chromosome names in a GTF file (e.g. 'chr' for using\nENSMEBL annotations with UCSC geneomes)\n",
                    "id": "#star-repeatmapping.cwl/sjdbGTFchrPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfeatureExon"
                    },
                    "doc": "exon\nstring: feature type in GTF file to be used as exons for building\ntranscripts\n",
                    "id": "#star-repeatmapping.cwl/sjdbGTFfeatureExon"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfile"
                    },
                    "doc": "string: path to the GTF file with annotations\n",
                    "id": "#star-repeatmapping.cwl/sjdbGTFfile"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentGene"
                    },
                    "doc": "gene_id\nstring: tag name to be used as exons' gene-parents (default \"gene_id\" works for GTF files)\n",
                    "id": "#star-repeatmapping.cwl/sjdbGTFtagExonParentGene"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentTranscript"
                    },
                    "doc": "transcript_id\nstring: tag name to be used as exons' transcript-parents (default\n\"transcript_id\" works for GTF files)\n",
                    "id": "#star-repeatmapping.cwl/sjdbGTFtagExonParentTranscript"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbOverhang"
                    },
                    "doc": "100\nint>0: length of the donor/acceptor sequence on each side of the junctions, ideally = (mate_length - 1)\n",
                    "id": "#star-repeatmapping.cwl/sjdbOverhang"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbScore"
                    },
                    "doc": "2\nint: extra alignment score for alignmets that cross database junctions\n",
                    "id": "#star-repeatmapping.cwl/sjdbScore"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sysShell"
                    },
                    "doc": "string: path to the shell binary, preferrably bash, e.g. /bin/bash.\n- ... the default shell is executed, typically /bin/sh. This was reported to fail on some Ubuntu systems - then you need to specify path to bash.\n",
                    "id": "#star-repeatmapping.cwl/sysShell"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopass1readsN"
                    },
                    "doc": "int: number of reads to process for the 1st step. Use very large number (or\ndefault -1) to map all reads in the first step.\n",
                    "id": "#star-repeatmapping.cwl/twopass1readsN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopassMode"
                    },
                    "doc": "None\nstring: 2-pass mapping mode.\nNone        ... 1-pass mapping\nBasic       ... basic 2-pass mapping, with all 1st pass junctions inserted into the genome indices on the fly\n",
                    "id": "#star-repeatmapping.cwl/twopassMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorDistNbins"
                    },
                    "doc": "9\nint>0: max number of bins between two anchors that allows aggregation of anchors into one window\n",
                    "id": "#star-repeatmapping.cwl/winAnchorDistNbins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorMultimapNmax"
                    },
                    "doc": "50\nint>0: max number of loci anchors are allowed to map to\n",
                    "id": "#star-repeatmapping.cwl/winAnchorMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winBinNbits"
                    },
                    "doc": "16\nint>0: =log2(winBin), where winBin is the size of the bin for the windows/clustering, each window will occupy an integer number of bins.\n",
                    "id": "#star-repeatmapping.cwl/winBinNbits"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winFlankNbins"
                    },
                    "doc": "4\nint>0: log2(winFlank), where win Flank is the size of the left and right flanking regions for each window\n",
                    "id": "#star-repeatmapping.cwl/winFlankNbins"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Aligned.out.bam"
                    },
                    "id": "#star-repeatmapping.cwl/aligned"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Log.final.out"
                    },
                    "id": "#star-repeatmapping.cwl/mappingstats"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate1"
                    },
                    "id": "#star-repeatmapping.cwl/output_map_unmapped_fwd"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate2"
                    },
                    "id": "#star-repeatmapping.cwl/output_map_unmapped_rev"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Log.out"
                    },
                    "id": "#star-repeatmapping.cwl/starsettings"
                }
            ],
            "baseCommand": [
                "STAR"
            ],
            "s:mainEntity": {
                "class": "s:SoftwareSourceCode",
                "s:name": "STAR",
                "s:about": "Aligns RNA-seq reads to a reference genome using uncompressed suffix arrays. STAR has a potential for accurately aligning long (several kilobases) reads that are emerging from the third-generation sequencing technologies.\n",
                "s:url": "https://github.com/alexdobin/STAR",
                "s:codeRepository": "https://github.com/alexdobin/STAR.git",
                "s:license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "s:targetProduct": {
                    "class": "s:SoftwareApplication",
                    "s:softwareVersion": "2.5.0b",
                    "s:applicationCategory": "commandline tool"
                },
                "s:programmingLanguage": "C++",
                "s:publication": [
                    {
                        "$import": "#bts635"
                    }
                ],
                "s:author": [
                    {
                        "$import": "#dobin@cshl.edu"
                    }
                ]
            },
            "s:downloadUrl": "https://github.com/common-workflow-language/workflows/blob/master/tools/STAR.cwl",
            "s:codeRepository": "https://github.com/common-workflow-language/workflows",
            "s:isPartOf": {
                "class": "s:CreativeWork",
                "s:name": "Common Workflow Language",
                "s:url": "http://commonwl.org/"
            },
            "s:author": {
                "class": "s:Person",
                "s:name": "Andrey Kartashov",
                "s:email": "mailto:Andrey.Kartashov@cchmc.org",
                "s:sameAs": [
                    {
                        "$import": "#0000-0001-9102-5681"
                    }
                ],
                "s:worksFor": [
                    {
                        "class": "s:Organization",
                        "s:name": "Cincinnati Children's Hospital Medical Center",
                        "s:location": "3333 Burnet Ave, Cincinnati, OH 45229-3026",
                        "s:department": [
                            {
                                "class": "s:Organization",
                                "s:name": "Barski Lab"
                            }
                        ]
                    }
                ]
            },
            "id": "#star-repeatmapping.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "coresMin": 2
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/cutadapt:1.14"
                }
            ],
            "baseCommand": [
                "cutadapt"
            ],
            "inputs": [
                {
                    "type": "string",
                    "default": "0.1",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-e"
                    },
                    "id": "#trim_pe.cwl/error_rate"
                },
                {
                    "type": "string",
                    "default": "fastq",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "id": "#trim_pe.cwl/f"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 13
                    },
                    "id": "#trim_pe.cwl/input_trim"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-A ",
                            "separate": false
                        }
                    },
                    "inputBinding": {
                        "position": 11
                    },
                    "id": "#trim_pe.cwl/input_trim_A_adapters"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-a ",
                            "separate": false
                        }
                    },
                    "inputBinding": {
                        "position": 12
                    },
                    "id": "#trim_pe.cwl/input_trim_a_adapters"
                },
                {
                    "default": [],
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-b ",
                            "separate": false
                        }
                    },
                    "inputBinding": {
                        "position": 9
                    },
                    "id": "#trim_pe.cwl/input_trim_b_adapters"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-g ",
                            "separate": false
                        }
                    },
                    "inputBinding": {
                        "position": 10
                    },
                    "id": "#trim_pe.cwl/input_trim_g_adapters"
                },
                {
                    "type": "string",
                    "default": "5",
                    "inputBinding": {
                        "position": 0,
                        "prefix": "-O"
                    },
                    "id": "#trim_pe.cwl/input_trim_overlap_length"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--match-read-wildcards"
                    },
                    "id": "#trim_pe.cwl/match_read_wildcards"
                },
                {
                    "type": "string",
                    "default": "18",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "-m"
                    },
                    "id": "#trim_pe.cwl/minimum_length"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-o",
                        "valueFrom": "${\n  if (inputs.output_r1 == \"\") {\n    return inputs.input_trim[0].nameroot + \"Tr.fq\";\n  }\n  else {\n    return inputs.output_r1;\n  }\n}\n"
                    },
                    "default": "",
                    "id": "#trim_pe.cwl/output_r1"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-p",
                        "valueFrom": "${\n  if (inputs.output_r2 == \"\") {\n    return inputs.input_trim[1].nameroot + \"Tr.fq\";\n  }\n  else {\n    return inputs.output_r2;\n  }\n}\n"
                    },
                    "default": "",
                    "id": "#trim_pe.cwl/output_r2"
                },
                {
                    "type": "string",
                    "default": "6",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--quality-cutoff"
                    },
                    "id": "#trim_pe.cwl/quality_cutoff"
                },
                {
                    "type": "string",
                    "default": "1",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--times"
                    },
                    "id": "#trim_pe.cwl/times"
                }
            ],
            "stdout": "$(inputs.input_trim[0].nameroot)Tr.metrics",
            "outputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_r1 == \"\") {\n    return [\n      inputs.input_trim[0].nameroot + \"Tr.fq\",\n      inputs.input_trim[1].nameroot + \"Tr.fq\"\n    ];\n  }\n  else {\n    return [\n      inputs.output_r1,\n      inputs.output_r2\n    ];\n  }\n}\n"
                    },
                    "id": "#trim_pe.cwl/output_trim"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.metrics"
                    },
                    "id": "#trim_pe.cwl/output_trim_report"
                }
            ],
            "doc": "This tool wraps cutadapt with default parameters set to paired-end eCLIP processing defaults.\n  Usage: cutadapt -a ADAPT1 -A ADAPT2 [options] -o out1.fastq -p out2.fastq in1.fastq in2.fastq\n",
            "id": "#trim_pe.cwl"
        },
        {
            "doc": "Workflow for handling reads containing one barcode.\nReturns the bam file containing read2 only.\n\nNotes:\n\n  runs the following steps: \n  - demultiplex\n  - trimfirst_file2string\n  - trimagain_file2string\n  - b1_trim_and_map\n  - view_r2\n  - index_r2_bam\n  - make_bigwigs\n",
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/barcodesfasta"
                },
                {
                    "type": "File",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/chrom_sizes"
                },
                {
                    "type": "string",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/dataset"
                },
                {
                    "type": "string",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/randomer_length"
                },
                {
                    "type": {
                        "type": "record",
                        "fields": [
                            {
                                "type": {
                                    "type": "array",
                                    "items": "string"
                                },
                                "name": "#wf_clipseqcore_pe_1barcode.cwl/read/barcodeids"
                            },
                            {
                                "type": "string",
                                "name": "#wf_clipseqcore_pe_1barcode.cwl/read/name"
                            },
                            {
                                "type": "File",
                                "name": "#wf_clipseqcore_pe_1barcode.cwl/read/read1"
                            },
                            {
                                "type": "File",
                                "name": "#wf_clipseqcore_pe_1barcode.cwl/read/read2"
                            }
                        ]
                    },
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/read"
                },
                {
                    "type": "Directory",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/repeatElementGenomeDir"
                },
                {
                    "type": "Directory",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/speciesGenomeDir"
                }
            ],
            "outputs": [
                {
                    "label": "Barcode1 read1 demultiplexed fastq",
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/A_output_demuxed_read1",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_demuxed_fastq_r1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/A_output_demuxed_read2",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_demuxed_fastq_r2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_mapgenome_mapped_to_genome",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_mapgenome_star_settings",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_mapgenome_stats",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_maprepeats_mapped_to_genome",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_maprepeats_star_settings",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_maprepeats_stats",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_maprepeats_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_barcodecollapsepe_bam",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_output_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_barcodecollapsepe_metrics",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_output_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_sorted_bam",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_output_prermdup_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_sorted_bam",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_output_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_sort_repunmapped_fastq",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_sorted_unmapped_fastq"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx1_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R1",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx1_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R2",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx1_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R1",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx1_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R2",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx1_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_metrics",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx1_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx2_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R1",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx2_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R2",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx2_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R1",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx2_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R2",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx2_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_metrics",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trimx2_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs/negbw",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/output_neg_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs/posbw",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/output_pos_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_1barcode.cwl/view_r2/output",
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/output_r2_bam"
                }
            ],
            "steps": [
                {
                    "run": "#wf_trim_and_map_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_A_adapters",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_a_adapters",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/a_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_a_adapters_default",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/a_adapters_default"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_g_adapters",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/g_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_g_adapters_default",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/g_adapters_default"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/A_output_demuxed_read1",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/read1"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/A_output_demuxed_read2",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/read2"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/repeatElementGenomeDir",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/repeatElementGenomeDir"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/speciesGenomeDir",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/speciesGenomeDir"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/trimagain_file2string/output",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/trimagain_overlap_length"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/trimfirst_file2string/output",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/trimfirst_overlap_length"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_metrics",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R1",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R1",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R2",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R2",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_metrics",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R1",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R1",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R2",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R2",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_maprepeats_mapped_to_genome",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_maprepeats_stats",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_maprepeats_star_settings",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_sort_repunmapped_fastq",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_mapgenome_mapped_to_genome",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_mapgenome_stats",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_mapgenome_star_settings",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/A_output_sorted_bam",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_barcodecollapsepe_bam",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_barcodecollapsepe_metrics",
                        "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_sorted_bam"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map"
                },
                {
                    "run": "#wf_demultiplex_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/barcodesfasta",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/barcodesfasta"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/dataset",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/dataset"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/randomer_length",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/randomer_length"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/read",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/read"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/A_output_demuxed_read1",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/A_output_demuxed_read2",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/B_output_demuxed_read1",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/B_output_demuxed_read2",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_output_trimfirst_overlap_length",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_output_trimagain_overlap_length",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_g_adapters",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_g_adapters_default",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_a_adapters",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_a_adapters_default",
                        "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_A_adapters"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex"
                },
                {
                    "run": "#samtools-index.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/view_r2/output",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/index_r2_bam/alignments"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/index_r2_bam/alignments_with_index"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/index_r2_bam"
                },
                {
                    "run": "#makebigwigfiles_PE.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/index_r2_bam/alignments_with_index",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs/bam"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/chrom_sizes",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs/chromsizes"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs/posbw",
                        "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs/negbw"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/make_bigwigs"
                },
                {
                    "run": "#file2string.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_output_trimagain_overlap_length",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/trimagain_file2string/file"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/trimagain_file2string/output"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/trimagain_file2string"
                },
                {
                    "run": "#file2string.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/demultiplex/AB_output_trimfirst_overlap_length",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/trimfirst_file2string/file"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/trimfirst_file2string/output"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/trimfirst_file2string"
                },
                {
                    "run": "#samtools-viewr2.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_1barcode.cwl/b1_trim_and_map/X_output_sorted_bam",
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/view_r2/input"
                        },
                        {
                            "default": true,
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/view_r2/isbam"
                        },
                        {
                            "default": 128,
                            "id": "#wf_clipseqcore_pe_1barcode.cwl/view_r2/readswithbits"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_1barcode.cwl/view_r2/output"
                    ],
                    "id": "#wf_clipseqcore_pe_1barcode.cwl/view_r2"
                }
            ],
            "id": "#wf_clipseqcore_pe_1barcode.cwl"
        },
        {
            "doc": "Workflow for handling reads containing two barcodes.\nReturns the bam file containing read2 only.\n\nNotes:\n\n  runs the following steps: \n  - demultiplex\n  - trimfirst_file2string\n  - trimagain_file2string\n  - b1_trim_and_map\n  - view_r2\n  - index_r2_bam\n  - make_bigwigs\n",
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/barcodesfasta"
                },
                {
                    "type": "File",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/chrom_sizes"
                },
                {
                    "type": "string",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/dataset"
                },
                {
                    "type": "string",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/randomer_length"
                },
                {
                    "type": {
                        "type": "record",
                        "fields": [
                            {
                                "type": {
                                    "type": "array",
                                    "items": "string"
                                },
                                "name": "#wf_clipseqcore_pe_2barcodes.cwl/read/barcodeids"
                            },
                            {
                                "type": "string",
                                "name": "#wf_clipseqcore_pe_2barcodes.cwl/read/name"
                            },
                            {
                                "type": "File",
                                "name": "#wf_clipseqcore_pe_2barcodes.cwl/read/read1"
                            },
                            {
                                "type": "File",
                                "name": "#wf_clipseqcore_pe_2barcodes.cwl/read/read2"
                            }
                        ]
                    },
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/read"
                },
                {
                    "type": "Directory",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/repeatElementGenomeDir"
                },
                {
                    "type": "Directory",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/speciesGenomeDir"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/A_output_demuxed_read1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_demuxed_fastq_r1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/A_output_demuxed_read2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_demuxed_fastq_r2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_mapgenome_mapped_to_genome",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_mapgenome_star_settings",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_mapgenome_stats",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_maprepeats_mapped_to_genome",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_maprepeats_star_settings",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_maprepeats_stats",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_maprepeats_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_barcodecollapsepe_bam",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_output_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_barcodecollapsepe_metrics",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_output_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_sorted_bam",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_output_prermdup_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_sorted_bam",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_output_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_sort_repunmapped_fastq",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_sorted_unmapped_fastq"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx1_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx1_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx1_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx1_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx1_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_metrics",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx1_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx2_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx2_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx2_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx2_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx2_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_metrics",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trimx2_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/B_output_demuxed_read1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_demuxed_fastq_r1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/B_output_demuxed_read2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_demuxed_fastq_r2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_mapgenome_mapped_to_genome",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_mapgenome_star_settings",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_mapgenome_stats",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_maprepeats_mapped_to_genome",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_maprepeats_star_settings",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_maprepeats_stats",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_maprepeats_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_barcodecollapsepe_bam",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_output_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_barcodecollapsepe_metrics",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_output_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_sorted_bam",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_output_prermdup_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_sorted_bam",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_output_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_sort_repunmapped_fastq",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_sorted_unmapped_fastq"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx1_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_report_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx1_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_report_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx1_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_stats_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx1_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_stats_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx1_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_metrics",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx1_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx2_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_report_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx2_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_report_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx2_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_stats_R1",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx2_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_stats_R2",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx2_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_metrics",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trimx2_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs/negbw",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/output_neg_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs/posbw",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/output_pos_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_clipseqcore_pe_2barcodes.cwl/view_r2/output",
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/output_r2_bam"
                }
            ],
            "steps": [
                {
                    "run": "#wf_trim_and_map_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_A_adapters",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_a_adapters",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/a_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_a_adapters_default",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/a_adapters_default"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_g_adapters",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/g_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_g_adapters_default",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/g_adapters_default"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/A_output_demuxed_read1",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/read1"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/A_output_demuxed_read2",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/read2"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/repeatElementGenomeDir",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/repeatElementGenomeDir"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/speciesGenomeDir",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/speciesGenomeDir"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/trimagain_file2string/output",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/trimagain_overlap_length"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/trimfirst_file2string/output",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/trimfirst_overlap_length"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_metrics",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_report_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_first_fastqc_stats_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_metrics",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_report_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_trim_again_fastqc_stats_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_maprepeats_mapped_to_genome",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_maprepeats_stats",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_maprepeats_star_settings",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_sort_repunmapped_fastq",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_mapgenome_mapped_to_genome",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_mapgenome_stats",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_mapgenome_star_settings",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/A_output_sorted_bam",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_barcodecollapsepe_bam",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_barcodecollapsepe_metrics",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_sorted_bam"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map"
                },
                {
                    "run": "#wf_trim_and_map_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_A_adapters",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_a_adapters",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/a_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_a_adapters_default",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/a_adapters_default"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_g_adapters",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/g_adapters"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_g_adapters_default",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/g_adapters_default"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/B_output_demuxed_read1",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/read1"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/B_output_demuxed_read2",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/read2"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/repeatElementGenomeDir",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/repeatElementGenomeDir"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/speciesGenomeDir",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/speciesGenomeDir"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/trimagain_file2string/output",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/trimagain_overlap_length"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/trimfirst_file2string/output",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/trimfirst_overlap_length"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_metrics",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_report_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_stats_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_report_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_first_fastqc_stats_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_metrics",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_report_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_stats_R1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_report_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_trim_again_fastqc_stats_R2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_maprepeats_mapped_to_genome",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_maprepeats_stats",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_maprepeats_star_settings",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_sort_repunmapped_fastq",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_mapgenome_mapped_to_genome",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_mapgenome_stats",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_mapgenome_star_settings",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/A_output_sorted_bam",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_barcodecollapsepe_bam",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_barcodecollapsepe_metrics",
                        "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_sorted_bam"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map"
                },
                {
                    "run": "#wf_demultiplex_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/barcodesfasta",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/barcodesfasta"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/dataset",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/dataset"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/randomer_length",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/randomer_length"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/read",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/read"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/A_output_demuxed_read1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/A_output_demuxed_read2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/B_output_demuxed_read1",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/B_output_demuxed_read2",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_output_trimfirst_overlap_length",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_output_trimagain_overlap_length",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_g_adapters",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_g_adapters_default",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_a_adapters",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_a_adapters_default",
                        "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_A_adapters"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex"
                },
                {
                    "run": "#samtools-index.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/view_r2/output",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/index_r2_bam/alignments"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/index_r2_bam/alignments_with_index"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/index_r2_bam"
                },
                {
                    "run": "#makebigwigfiles_PE.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/index_r2_bam/alignments_with_index",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs/bam"
                        },
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/chrom_sizes",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs/chromsizes"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs/posbw",
                        "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs/negbw"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/make_bigwigs"
                },
                {
                    "run": "#samtools-merge.cwl",
                    "in": [
                        {
                            "source": [
                                "#wf_clipseqcore_pe_2barcodes.cwl/b1_trim_and_map/X_output_sorted_bam",
                                "#wf_clipseqcore_pe_2barcodes.cwl/b2_trim_and_map/X_output_sorted_bam"
                            ],
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/merge/input_bam_files"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/merge/output_bam_file"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/merge"
                },
                {
                    "run": "#file2string.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_output_trimagain_overlap_length",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/trimagain_file2string/file"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/trimagain_file2string/output"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/trimagain_file2string"
                },
                {
                    "run": "#file2string.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/demultiplex/AB_output_trimfirst_overlap_length",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/trimfirst_file2string/file"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/trimfirst_file2string/output"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/trimfirst_file2string"
                },
                {
                    "run": "#samtools-viewr2.cwl",
                    "in": [
                        {
                            "source": "#wf_clipseqcore_pe_2barcodes.cwl/merge/output_bam_file",
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/view_r2/input"
                        },
                        {
                            "default": true,
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/view_r2/isbam"
                        },
                        {
                            "default": 128,
                            "id": "#wf_clipseqcore_pe_2barcodes.cwl/view_r2/readswithbits"
                        }
                    ],
                    "out": [
                        "#wf_clipseqcore_pe_2barcodes.cwl/view_r2/output"
                    ],
                    "id": "#wf_clipseqcore_pe_2barcodes.cwl/view_r2"
                }
            ],
            "id": "#wf_clipseqcore_pe_2barcodes.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#wf_demultiplex_pe.cwl/barcodesfasta"
                },
                {
                    "type": "string",
                    "id": "#wf_demultiplex_pe.cwl/dataset"
                },
                {
                    "type": "string",
                    "id": "#wf_demultiplex_pe.cwl/randomer_length"
                },
                {
                    "type": {
                        "type": "record",
                        "fields": [
                            {
                                "type": {
                                    "type": "array",
                                    "items": "string"
                                },
                                "name": "#wf_demultiplex_pe.cwl/read/barcodeids"
                            },
                            {
                                "type": "string",
                                "name": "#wf_demultiplex_pe.cwl/read/name"
                            },
                            {
                                "type": "File",
                                "name": "#wf_demultiplex_pe.cwl/read/read1"
                            },
                            {
                                "type": "File",
                                "name": "#wf_demultiplex_pe.cwl/read/read2"
                            }
                        ]
                    },
                    "id": "#wf_demultiplex_pe.cwl/read"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/A_adapters",
                    "id": "#wf_demultiplex_pe.cwl/AB_A_adapters"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/a_adapters",
                    "id": "#wf_demultiplex_pe.cwl/AB_a_adapters"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/a_adapters_default",
                    "id": "#wf_demultiplex_pe.cwl/AB_a_adapters_default"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/g_adapters",
                    "id": "#wf_demultiplex_pe.cwl/AB_g_adapters"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/g_adapters_default",
                    "id": "#wf_demultiplex_pe.cwl/AB_g_adapters_default"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/trimagain_overlap_length",
                    "id": "#wf_demultiplex_pe.cwl/AB_output_trimagain_overlap_length"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/trimfirst_overlap_length",
                    "id": "#wf_demultiplex_pe.cwl/AB_output_trimfirst_overlap_length"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_demux/demuxedAfwd",
                    "id": "#wf_demultiplex_pe.cwl/A_output_demuxed_read1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_demux/demuxedArev",
                    "id": "#wf_demultiplex_pe.cwl/A_output_demuxed_read2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_demux/demuxedBfwd",
                    "id": "#wf_demultiplex_pe.cwl/B_output_demuxed_read1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_demultiplex_pe.cwl/AB_demux/demuxedBrev",
                    "id": "#wf_demultiplex_pe.cwl/B_output_demuxed_read2"
                }
            ],
            "steps": [
                {
                    "run": "#demux_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_demultiplex_pe.cwl/barcodesfasta",
                            "id": "#wf_demultiplex_pe.cwl/AB_demux/barcodesfasta"
                        },
                        {
                            "source": "#wf_demultiplex_pe.cwl/dataset",
                            "id": "#wf_demultiplex_pe.cwl/AB_demux/dataset"
                        },
                        {
                            "source": "#wf_demultiplex_pe.cwl/randomer_length",
                            "id": "#wf_demultiplex_pe.cwl/AB_demux/randomer_length"
                        },
                        {
                            "source": "#wf_demultiplex_pe.cwl/read",
                            "id": "#wf_demultiplex_pe.cwl/AB_demux/reads"
                        }
                    ],
                    "out": [
                        "#wf_demultiplex_pe.cwl/AB_demux/demuxedAfwd",
                        "#wf_demultiplex_pe.cwl/AB_demux/demuxedArev",
                        "#wf_demultiplex_pe.cwl/AB_demux/demuxedBfwd",
                        "#wf_demultiplex_pe.cwl/AB_demux/demuxedBrev",
                        "#wf_demultiplex_pe.cwl/AB_demux/output_demuxedpairedend_metrics",
                        "#wf_demultiplex_pe.cwl/AB_demux/output_dataset",
                        "#wf_demultiplex_pe.cwl/AB_demux/name",
                        "#wf_demultiplex_pe.cwl/AB_demux/barcodeidA",
                        "#wf_demultiplex_pe.cwl/AB_demux/barcodeidB"
                    ],
                    "id": "#wf_demultiplex_pe.cwl/AB_demux"
                },
                {
                    "run": "#parsebarcodes.cwl",
                    "in": [
                        {
                            "source": "#wf_demultiplex_pe.cwl/AB_demux/barcodeidA",
                            "id": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/barcodeidA"
                        },
                        {
                            "source": "#wf_demultiplex_pe.cwl/AB_demux/barcodeidB",
                            "id": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/barcodeidB"
                        },
                        {
                            "source": "#wf_demultiplex_pe.cwl/barcodesfasta",
                            "id": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/barcodesfasta"
                        },
                        {
                            "source": "#wf_demultiplex_pe.cwl/randomer_length",
                            "id": "#wf_demultiplex_pe.cwl/AB_parsebarcodes/randomer_length"
                        }
                    ],
                    "out": [
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/trimfirst_overlap_length",
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/trimagain_overlap_length",
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/g_adapters_default",
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/a_adapters_default",
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/g_adapters",
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/a_adapters",
                        "#wf_demultiplex_pe.cwl/AB_parsebarcodes/A_adapters"
                    ],
                    "id": "#wf_demultiplex_pe.cwl/AB_parsebarcodes"
                }
            ],
            "id": "#wf_demultiplex_pe.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#wf_fastqc.cwl/reads"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#wf_fastqc.cwl/step_rename_report/outfile",
                    "id": "#wf_fastqc.cwl/output_qc_report"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_fastqc.cwl/step_rename_stats/outfile",
                    "id": "#wf_fastqc.cwl/output_qc_stats"
                }
            ],
            "steps": [
                {
                    "run": "#fastqc.cwl",
                    "in": [
                        {
                            "source": "#wf_fastqc.cwl/reads",
                            "id": "#wf_fastqc.cwl/step_fastqc/reads"
                        }
                    ],
                    "out": [
                        "#wf_fastqc.cwl/step_fastqc/output_qc_report",
                        "#wf_fastqc.cwl/step_fastqc/output_qc_stats"
                    ],
                    "id": "#wf_fastqc.cwl/step_fastqc"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#wf_fastqc.cwl/reads",
                            "valueFrom": "${ return self.nameroot + \".fastqc_report\"; }",
                            "id": "#wf_fastqc.cwl/step_rename_report/newname"
                        },
                        {
                            "source": "#wf_fastqc.cwl/step_fastqc/output_qc_report",
                            "id": "#wf_fastqc.cwl/step_rename_report/srcfile"
                        },
                        {
                            "default": ".html",
                            "id": "#wf_fastqc.cwl/step_rename_report/suffix"
                        }
                    ],
                    "out": [
                        "#wf_fastqc.cwl/step_rename_report/outfile"
                    ],
                    "id": "#wf_fastqc.cwl/step_rename_report"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#wf_fastqc.cwl/reads",
                            "valueFrom": "${ return self.nameroot + \".fastqc_data\"; }",
                            "id": "#wf_fastqc.cwl/step_rename_stats/newname"
                        },
                        {
                            "source": "#wf_fastqc.cwl/step_fastqc/output_qc_stats",
                            "id": "#wf_fastqc.cwl/step_rename_stats/srcfile"
                        },
                        {
                            "default": ".txt",
                            "id": "#wf_fastqc.cwl/step_rename_stats/suffix"
                        }
                    ],
                    "out": [
                        "#wf_fastqc.cwl/step_rename_stats/outfile"
                    ],
                    "id": "#wf_fastqc.cwl/step_rename_stats"
                }
            ],
            "doc": "This workflow takes in single-end reads, and performs the following steps in order:\ndemux_se.cwl (does not actually demux for single end, but mirrors the paired-end processing protocol)\n",
            "id": "#wf_fastqc.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#main/barcodesfasta"
                },
                {
                    "type": "File",
                    "id": "#main/chrom_sizes"
                },
                {
                    "type": "string",
                    "id": "#main/dataset"
                },
                {
                    "type": "string",
                    "id": "#main/randomer_length"
                },
                {
                    "type": "Directory",
                    "id": "#main/repeatElementGenomeDir"
                },
                {
                    "type": {
                        "type": "array",
                        "items": {
                            "type": "record",
                            "fields": [
                                {
                                    "type": {
                                        "type": "array",
                                        "items": "string"
                                    },
                                    "name": "#main/sample/barcodeids"
                                },
                                {
                                    "type": "string",
                                    "name": "#main/sample/name"
                                },
                                {
                                    "type": "File",
                                    "name": "#main/sample/read1"
                                },
                                {
                                    "type": "File",
                                    "name": "#main/sample/read2"
                                }
                            ]
                        }
                    },
                    "id": "#main/sample"
                },
                {
                    "type": "string",
                    "id": "#main/species"
                },
                {
                    "type": "Directory",
                    "id": "#main/speciesGenomeDir"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/step_clipper/output_bed",
                    "id": "#main/output_clipper_bed"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_compress_peaks/output_bed",
                    "id": "#main/output_compressed_peaks"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_calculate_entropy/output_entropynum",
                    "id": "#main/output_entropynum"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_output_barcodecollapsepe_bam",
                    "id": "#main/output_input_b1_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_output_barcodecollapsepe_metrics",
                    "id": "#main/output_input_b1_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_demuxed_fastq_r1",
                    "id": "#main/output_input_b1_demuxed_fastq_r1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_demuxed_fastq_r2",
                    "id": "#main/output_input_b1_demuxed_fastq_r2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_mapgenome_mapped_to_genome",
                    "id": "#main/output_input_b1_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_mapgenome_star_settings",
                    "id": "#main/output_input_b1_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_mapgenome_stats",
                    "id": "#main/output_input_b1_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_maprepeats_mapped_to_genome",
                    "id": "#main/output_input_b1_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_maprepeats_star_settings",
                    "id": "#main/output_input_b1_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_maprepeats_stats",
                    "id": "#main/output_input_b1_maprepeats_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_output_sorted_bam",
                    "id": "#main/output_input_b1_output_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_output_prermdup_sorted_bam",
                    "id": "#main/output_input_b1_prermdup_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_input_alignment/b1_sorted_unmapped_fastq",
                    "id": "#main/output_input_b1_sorted_unmapped_fastq"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_input_alignment/b1_trimx1_fastq",
                    "id": "#main/output_input_b1_trimx1_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx1_fastqc_report_R1",
                    "id": "#main/output_input_b1_trimx1_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx1_fastqc_report_R2",
                    "id": "#main/output_input_b1_trimx1_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx1_fastqc_stats_R1",
                    "id": "#main/output_input_b1_trimx1_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx1_fastqc_stats_R2",
                    "id": "#main/output_input_b1_trimx1_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx1_metrics",
                    "id": "#main/output_input_b1_trimx1_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_input_alignment/b1_trimx2_fastq",
                    "id": "#main/output_input_b1_trimx2_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx2_fastqc_report_R1",
                    "id": "#main/output_input_b1_trimx2_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx2_fastqc_report_R2",
                    "id": "#main/output_input_b1_trimx2_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx2_fastqc_stats_R1",
                    "id": "#main/output_input_b1_trimx2_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx2_fastqc_stats_R2",
                    "id": "#main/output_input_b1_trimx2_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/b1_trimx2_metrics",
                    "id": "#main/output_input_b1_trimx2_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/output_r2_bam",
                    "id": "#main/output_input_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/output_neg_bw",
                    "id": "#main/output_input_neg_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_alignment/output_pos_bw",
                    "id": "#main/output_input_pos_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_input_normalize_peaks/inputnormedBed",
                    "id": "#main/output_inputnormed_peaks"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_output_barcodecollapsepe_bam",
                    "id": "#main/output_ip_b1_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_output_barcodecollapsepe_metrics",
                    "id": "#main/output_ip_b1_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_demuxed_fastq_r1",
                    "id": "#main/output_ip_b1_demuxed_fastq_r1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_demuxed_fastq_r2",
                    "id": "#main/output_ip_b1_demuxed_fastq_r2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_mapgenome_mapped_to_genome",
                    "id": "#main/output_ip_b1_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_mapgenome_star_settings",
                    "id": "#main/output_ip_b1_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_mapgenome_stats",
                    "id": "#main/output_ip_b1_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_maprepeats_mapped_to_genome",
                    "id": "#main/output_ip_b1_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_maprepeats_star_settings",
                    "id": "#main/output_ip_b1_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_maprepeats_stats",
                    "id": "#main/output_ip_b1_maprepeats_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_output_sorted_bam",
                    "id": "#main/output_ip_b1_output_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_output_prermdup_sorted_bam",
                    "id": "#main/output_ip_b1_prermdup_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_ip_alignment/b1_sorted_unmapped_fastq",
                    "id": "#main/output_ip_b1_sorted_unmapped_fastq"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_ip_alignment/b1_trimx1_fastq",
                    "id": "#main/output_ip_b1_trimx1_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx1_fastqc_report_R1",
                    "id": "#main/output_ip_b1_trimx1_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx1_fastqc_report_R2",
                    "id": "#main/output_ip_b1_trimx1_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx1_fastqc_stats_R1",
                    "id": "#main/output_ip_b1_trimx1_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx1_fastqc_stats_R2",
                    "id": "#main/output_ip_b1_trimx1_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx1_metrics",
                    "id": "#main/output_ip_b1_trimx1_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_ip_alignment/b1_trimx2_fastq",
                    "id": "#main/output_ip_b1_trimx2_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx2_fastqc_report_R1",
                    "id": "#main/output_ip_b1_trimx2_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx2_fastqc_report_R2",
                    "id": "#main/output_ip_b1_trimx2_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx2_fastqc_stats_R1",
                    "id": "#main/output_ip_b1_trimx2_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx2_fastqc_stats_R2",
                    "id": "#main/output_ip_b1_trimx2_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b1_trimx2_metrics",
                    "id": "#main/output_ip_b1_trimx2_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_output_barcodecollapsepe_bam",
                    "id": "#main/output_ip_b2_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_output_barcodecollapsepe_metrics",
                    "id": "#main/output_ip_b2_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_demuxed_fastq_r1",
                    "id": "#main/output_ip_b2_demuxed_fastq_r1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_demuxed_fastq_r2",
                    "id": "#main/output_ip_b2_demuxed_fastq_r2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_mapgenome_mapped_to_genome",
                    "id": "#main/output_ip_b2_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_mapgenome_star_settings",
                    "id": "#main/output_ip_b2_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_mapgenome_stats",
                    "id": "#main/output_ip_b2_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_maprepeats_mapped_to_genome",
                    "id": "#main/output_ip_b2_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_maprepeats_star_settings",
                    "id": "#main/output_ip_b2_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_maprepeats_stats",
                    "id": "#main/output_ip_b2_maprepeats_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_output_sorted_bam",
                    "id": "#main/output_ip_b2_output_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_output_prermdup_sorted_bam",
                    "id": "#main/output_ip_b2_prermdup_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_ip_alignment/b2_sorted_unmapped_fastq",
                    "id": "#main/output_ip_b2_sorted_unmapped_fastq"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_ip_alignment/b2_trimx1_fastq",
                    "id": "#main/output_ip_b2_trimx1_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx1_fastqc_report_R1",
                    "id": "#main/output_ip_b2_trimx1_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx1_fastqc_report_R2",
                    "id": "#main/output_ip_b2_trimx1_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx1_fastqc_stats_R1",
                    "id": "#main/output_ip_b2_trimx1_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx1_fastqc_stats_R2",
                    "id": "#main/output_ip_b2_trimx1_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx1_metrics",
                    "id": "#main/output_ip_b2_trimx1_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/step_ip_alignment/b2_trimx2_fastq",
                    "id": "#main/output_ip_b2_trimx2_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx2_fastqc_report_R1",
                    "id": "#main/output_ip_b2_trimx2_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx2_fastqc_report_R2",
                    "id": "#main/output_ip_b2_trimx2_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx2_fastqc_stats_R1",
                    "id": "#main/output_ip_b2_trimx2_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx2_fastqc_stats_R2",
                    "id": "#main/output_ip_b2_trimx2_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/b2_trimx2_metrics",
                    "id": "#main/output_ip_b2_trimx2_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/output_r2_bam",
                    "id": "#main/output_ip_merged_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/output_neg_bw",
                    "id": "#main/output_ip_neg_bw"
                },
                {
                    "type": "File",
                    "outputSource": "#main/step_ip_alignment/output_pos_bw",
                    "id": "#main/output_ip_pos_bw"
                }
            ],
            "steps": [
                {
                    "run": "#calculate_entropy.cwl",
                    "in": [
                        {
                            "source": "#main/step_input_normalize_peaks/inputnormedBedfull",
                            "id": "#main/step_calculate_entropy/full"
                        },
                        {
                            "source": "#main/step_input_mapped_readnum/output",
                            "id": "#main/step_calculate_entropy/input_mapped"
                        },
                        {
                            "source": "#main/step_ip_mapped_readnum/output",
                            "id": "#main/step_calculate_entropy/ip_mapped"
                        }
                    ],
                    "out": [
                        "#main/step_calculate_entropy/output_entropynum"
                    ],
                    "id": "#main/step_calculate_entropy"
                },
                {
                    "run": "#clipper.cwl",
                    "in": [
                        {
                            "source": "#main/step_index_ip/alignments_with_index",
                            "id": "#main/step_clipper/bam"
                        },
                        {
                            "default": "",
                            "id": "#main/step_clipper/outfile"
                        },
                        {
                            "source": "#main/species",
                            "id": "#main/step_clipper/species"
                        }
                    ],
                    "out": [
                        "#main/step_clipper/output_bed"
                    ],
                    "id": "#main/step_clipper"
                },
                {
                    "run": "#peakscompress.cwl",
                    "in": [
                        {
                            "source": "#main/step_input_normalize_peaks/inputnormedBed",
                            "id": "#main/step_compress_peaks/input_bed"
                        }
                    ],
                    "out": [
                        "#main/step_compress_peaks/output_bed"
                    ],
                    "id": "#main/step_compress_peaks"
                },
                {
                    "run": "#samtools-index.cwl",
                    "in": [
                        {
                            "source": "#main/step_input_alignment/output_r2_bam",
                            "id": "#main/step_index_input/alignments"
                        }
                    ],
                    "out": [
                        "#main/step_index_input/alignments_with_index"
                    ],
                    "id": "#main/step_index_input"
                },
                {
                    "run": "#samtools-index.cwl",
                    "in": [
                        {
                            "source": "#main/step_ip_alignment/output_r2_bam",
                            "id": "#main/step_index_ip/alignments"
                        }
                    ],
                    "out": [
                        "#main/step_index_ip/alignments_with_index"
                    ],
                    "id": "#main/step_index_ip"
                },
                {
                    "run": "#wf_clipseqcore_pe_1barcode.cwl",
                    "in": [
                        {
                            "source": "#main/barcodesfasta",
                            "id": "#main/step_input_alignment/barcodesfasta"
                        },
                        {
                            "source": "#main/chrom_sizes",
                            "id": "#main/step_input_alignment/chrom_sizes"
                        },
                        {
                            "source": "#main/dataset",
                            "id": "#main/step_input_alignment/dataset"
                        },
                        {
                            "source": "#main/randomer_length",
                            "id": "#main/step_input_alignment/randomer_length"
                        },
                        {
                            "source": "#main/sample",
                            "valueFrom": "${\n  return self[1];\n}\n",
                            "id": "#main/step_input_alignment/read"
                        },
                        {
                            "source": "#main/repeatElementGenomeDir",
                            "id": "#main/step_input_alignment/repeatElementGenomeDir"
                        },
                        {
                            "source": "#main/species",
                            "id": "#main/step_input_alignment/species"
                        },
                        {
                            "source": "#main/speciesGenomeDir",
                            "id": "#main/step_input_alignment/speciesGenomeDir"
                        }
                    ],
                    "out": [
                        "#main/step_input_alignment/b1_demuxed_fastq_r1",
                        "#main/step_input_alignment/b1_demuxed_fastq_r2",
                        "#main/step_input_alignment/b1_trimx1_fastq",
                        "#main/step_input_alignment/b1_trimx1_metrics",
                        "#main/step_input_alignment/b1_trimx1_fastqc_report_R1",
                        "#main/step_input_alignment/b1_trimx1_fastqc_stats_R1",
                        "#main/step_input_alignment/b1_trimx1_fastqc_report_R2",
                        "#main/step_input_alignment/b1_trimx1_fastqc_stats_R2",
                        "#main/step_input_alignment/b1_trimx2_fastq",
                        "#main/step_input_alignment/b1_trimx2_metrics",
                        "#main/step_input_alignment/b1_trimx2_fastqc_report_R1",
                        "#main/step_input_alignment/b1_trimx2_fastqc_stats_R1",
                        "#main/step_input_alignment/b1_trimx2_fastqc_report_R2",
                        "#main/step_input_alignment/b1_trimx2_fastqc_stats_R2",
                        "#main/step_input_alignment/b1_maprepeats_mapped_to_genome",
                        "#main/step_input_alignment/b1_maprepeats_stats",
                        "#main/step_input_alignment/b1_maprepeats_star_settings",
                        "#main/step_input_alignment/b1_sorted_unmapped_fastq",
                        "#main/step_input_alignment/b1_mapgenome_mapped_to_genome",
                        "#main/step_input_alignment/b1_mapgenome_stats",
                        "#main/step_input_alignment/b1_mapgenome_star_settings",
                        "#main/step_input_alignment/b1_output_prermdup_sorted_bam",
                        "#main/step_input_alignment/b1_output_barcodecollapsepe_bam",
                        "#main/step_input_alignment/b1_output_barcodecollapsepe_metrics",
                        "#main/step_input_alignment/b1_output_sorted_bam",
                        "#main/step_input_alignment/output_r2_bam",
                        "#main/step_input_alignment/output_pos_bw",
                        "#main/step_input_alignment/output_neg_bw"
                    ],
                    "id": "#main/step_input_alignment"
                },
                {
                    "run": "#samtools-mappedreadnum.cwl",
                    "in": [
                        {
                            "default": true,
                            "id": "#main/step_input_mapped_readnum/count"
                        },
                        {
                            "source": "#main/step_input_alignment/output_r2_bam",
                            "id": "#main/step_input_mapped_readnum/input"
                        },
                        {
                            "default": "input_mapped_readnum.txt",
                            "id": "#main/step_input_mapped_readnum/output_name"
                        },
                        {
                            "default": 4,
                            "id": "#main/step_input_mapped_readnum/readswithoutbits"
                        }
                    ],
                    "out": [
                        "#main/step_input_mapped_readnum/output"
                    ],
                    "id": "#main/step_input_mapped_readnum"
                },
                {
                    "run": "#overlap_peakfi_with_bam_PE.cwl",
                    "in": [
                        {
                            "source": "#main/step_index_ip/alignments_with_index",
                            "id": "#main/step_input_normalize_peaks/clipBamFile"
                        },
                        {
                            "source": "#main/step_ip_mapped_readnum/output",
                            "id": "#main/step_input_normalize_peaks/clipReadnum"
                        },
                        {
                            "source": "#main/step_index_input/alignments_with_index",
                            "id": "#main/step_input_normalize_peaks/inputBamFile"
                        },
                        {
                            "source": "#main/step_input_mapped_readnum/output",
                            "id": "#main/step_input_normalize_peaks/inputReadnum"
                        },
                        {
                            "source": "#main/step_clipper/output_bed",
                            "id": "#main/step_input_normalize_peaks/peakFile"
                        }
                    ],
                    "out": [
                        "#main/step_input_normalize_peaks/inputnormedBed",
                        "#main/step_input_normalize_peaks/inputnormedBedfull"
                    ],
                    "id": "#main/step_input_normalize_peaks"
                },
                {
                    "run": "#wf_clipseqcore_pe_2barcodes.cwl",
                    "in": [
                        {
                            "source": "#main/barcodesfasta",
                            "id": "#main/step_ip_alignment/barcodesfasta"
                        },
                        {
                            "source": "#main/chrom_sizes",
                            "id": "#main/step_ip_alignment/chrom_sizes"
                        },
                        {
                            "source": "#main/dataset",
                            "id": "#main/step_ip_alignment/dataset"
                        },
                        {
                            "source": "#main/randomer_length",
                            "id": "#main/step_ip_alignment/randomer_length"
                        },
                        {
                            "source": "#main/sample",
                            "valueFrom": "${\n  return self[0];\n}\n",
                            "id": "#main/step_ip_alignment/read"
                        },
                        {
                            "source": "#main/repeatElementGenomeDir",
                            "id": "#main/step_ip_alignment/repeatElementGenomeDir"
                        },
                        {
                            "source": "#main/species",
                            "id": "#main/step_ip_alignment/species"
                        },
                        {
                            "source": "#main/speciesGenomeDir",
                            "id": "#main/step_ip_alignment/speciesGenomeDir"
                        }
                    ],
                    "out": [
                        "#main/step_ip_alignment/b1_demuxed_fastq_r1",
                        "#main/step_ip_alignment/b1_demuxed_fastq_r2",
                        "#main/step_ip_alignment/b1_trimx1_fastq",
                        "#main/step_ip_alignment/b1_trimx1_metrics",
                        "#main/step_ip_alignment/b1_trimx1_fastqc_report_R1",
                        "#main/step_ip_alignment/b1_trimx1_fastqc_stats_R1",
                        "#main/step_ip_alignment/b1_trimx1_fastqc_report_R2",
                        "#main/step_ip_alignment/b1_trimx1_fastqc_stats_R2",
                        "#main/step_ip_alignment/b1_trimx2_fastq",
                        "#main/step_ip_alignment/b1_trimx2_metrics",
                        "#main/step_ip_alignment/b1_trimx2_fastqc_report_R1",
                        "#main/step_ip_alignment/b1_trimx2_fastqc_stats_R1",
                        "#main/step_ip_alignment/b1_trimx2_fastqc_report_R2",
                        "#main/step_ip_alignment/b1_trimx2_fastqc_stats_R2",
                        "#main/step_ip_alignment/b1_maprepeats_mapped_to_genome",
                        "#main/step_ip_alignment/b1_maprepeats_stats",
                        "#main/step_ip_alignment/b1_maprepeats_star_settings",
                        "#main/step_ip_alignment/b1_sorted_unmapped_fastq",
                        "#main/step_ip_alignment/b1_mapgenome_mapped_to_genome",
                        "#main/step_ip_alignment/b1_mapgenome_stats",
                        "#main/step_ip_alignment/b1_mapgenome_star_settings",
                        "#main/step_ip_alignment/b1_output_prermdup_sorted_bam",
                        "#main/step_ip_alignment/b1_output_barcodecollapsepe_bam",
                        "#main/step_ip_alignment/b1_output_barcodecollapsepe_metrics",
                        "#main/step_ip_alignment/b1_output_sorted_bam",
                        "#main/step_ip_alignment/b2_demuxed_fastq_r1",
                        "#main/step_ip_alignment/b2_demuxed_fastq_r2",
                        "#main/step_ip_alignment/b2_trimx1_fastq",
                        "#main/step_ip_alignment/b2_trimx1_metrics",
                        "#main/step_ip_alignment/b2_trimx1_fastqc_report_R1",
                        "#main/step_ip_alignment/b2_trimx1_fastqc_stats_R1",
                        "#main/step_ip_alignment/b2_trimx1_fastqc_report_R2",
                        "#main/step_ip_alignment/b2_trimx1_fastqc_stats_R2",
                        "#main/step_ip_alignment/b2_trimx2_fastq",
                        "#main/step_ip_alignment/b2_trimx2_metrics",
                        "#main/step_ip_alignment/b2_trimx2_fastqc_report_R1",
                        "#main/step_ip_alignment/b2_trimx2_fastqc_stats_R1",
                        "#main/step_ip_alignment/b2_trimx2_fastqc_report_R2",
                        "#main/step_ip_alignment/b2_trimx2_fastqc_stats_R2",
                        "#main/step_ip_alignment/b2_maprepeats_mapped_to_genome",
                        "#main/step_ip_alignment/b2_maprepeats_stats",
                        "#main/step_ip_alignment/b2_maprepeats_star_settings",
                        "#main/step_ip_alignment/b2_sorted_unmapped_fastq",
                        "#main/step_ip_alignment/b2_mapgenome_mapped_to_genome",
                        "#main/step_ip_alignment/b2_mapgenome_stats",
                        "#main/step_ip_alignment/b2_mapgenome_star_settings",
                        "#main/step_ip_alignment/b2_output_prermdup_sorted_bam",
                        "#main/step_ip_alignment/b2_output_barcodecollapsepe_bam",
                        "#main/step_ip_alignment/b2_output_barcodecollapsepe_metrics",
                        "#main/step_ip_alignment/b2_output_sorted_bam",
                        "#main/step_ip_alignment/output_r2_bam",
                        "#main/step_ip_alignment/output_pos_bw",
                        "#main/step_ip_alignment/output_neg_bw"
                    ],
                    "id": "#main/step_ip_alignment"
                },
                {
                    "run": "#samtools-mappedreadnum.cwl",
                    "in": [
                        {
                            "default": true,
                            "id": "#main/step_ip_mapped_readnum/count"
                        },
                        {
                            "source": "#main/step_ip_alignment/output_r2_bam",
                            "id": "#main/step_ip_mapped_readnum/input"
                        },
                        {
                            "default": "ip_mapped_readnum.txt",
                            "id": "#main/step_ip_mapped_readnum/output_name"
                        },
                        {
                            "default": 4,
                            "id": "#main/step_ip_mapped_readnum/readswithoutbits"
                        }
                    ],
                    "out": [
                        "#main/step_ip_mapped_readnum/output"
                    ],
                    "id": "#main/step_ip_mapped_readnum"
                }
            ],
            "id": "#main"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/A_adapters"
                },
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/a_adapters"
                },
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/a_adapters_default"
                },
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/g_adapters"
                },
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/g_adapters_default"
                },
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/read1"
                },
                {
                    "type": "File",
                    "id": "#wf_trim_and_map_pe.cwl/read2"
                },
                {
                    "type": "Directory",
                    "id": "#wf_trim_and_map_pe.cwl/repeatElementGenomeDir"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "id": "#wf_trim_and_map_pe.cwl/sort_names"
                },
                {
                    "type": "Directory",
                    "id": "#wf_trim_and_map_pe.cwl/speciesGenomeDir"
                },
                {
                    "type": "string",
                    "default": "0.1",
                    "id": "#wf_trim_and_map_pe.cwl/trim_error_rate"
                },
                {
                    "type": "string",
                    "default": "1",
                    "id": "#wf_trim_and_map_pe.cwl/trim_times"
                },
                {
                    "type": "string",
                    "id": "#wf_trim_and_map_pe.cwl/trimagain_overlap_length"
                },
                {
                    "type": "string",
                    "id": "#wf_trim_and_map_pe.cwl/trimfirst_overlap_length"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/rename_mapped_genome/outfile",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_mapgenome_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/A_map_genome/starsettings",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_mapgenome_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/A_map_genome/mappingstats",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_mapgenome_stats"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/rename_mapped_repeats/outfile",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_maprepeats_mapped_to_genome"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/A_map_repeats/starsettings",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_maprepeats_star_settings"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/A_map_repeats/mappingstats",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_maprepeats_stats"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_gzip_sort_repunmapped_fastq/gzipped",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_sort_repunmapped_fastq"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/X_sortlexico/output_sort_bam",
                    "id": "#wf_trim_and_map_pe.cwl/A_output_sorted_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe/output_barcodecollapsepe_bam",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_barcodecollapsepe_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe/output_barcodecollapsepe_metrics",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_barcodecollapsepe_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/X_sort/output_sort_bam",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_sorted_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again/gzipped",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_again"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R1/output_qc_report",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_again_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R2/output_qc_report",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_again_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R1/output_qc_stats",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_again_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R2/output_qc_stats",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_again_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/X_trim_again/output_trim_report",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_again_metrics"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim/gzipped",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_first"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R1/output_qc_report",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_first_fastqc_report_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R2/output_qc_report",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_first_fastqc_report_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R1/output_qc_stats",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_first_fastqc_stats_R1"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R2/output_qc_stats",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_first_fastqc_stats_R2"
                },
                {
                    "type": "File",
                    "outputSource": "#wf_trim_and_map_pe.cwl/X_trim/output_trim_report",
                    "id": "#wf_trim_and_map_pe.cwl/X_output_trim_first_metrics"
                }
            ],
            "steps": [
                {
                    "run": "#star-genome.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/speciesGenomeDir",
                            "id": "#wf_trim_and_map_pe.cwl/A_map_genome/genomeDir"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_sort_repunmapped_fastq/output_fastqsort_sortedfastq",
                            "id": "#wf_trim_and_map_pe.cwl/A_map_genome/readFilesIn"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/A_map_genome/aligned",
                        "#wf_trim_and_map_pe.cwl/A_map_genome/output_map_unmapped_fwd",
                        "#wf_trim_and_map_pe.cwl/A_map_genome/output_map_unmapped_rev",
                        "#wf_trim_and_map_pe.cwl/A_map_genome/starsettings",
                        "#wf_trim_and_map_pe.cwl/A_map_genome/mappingstats"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/A_map_genome"
                },
                {
                    "run": "#star-repeatmapping.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/repeatElementGenomeDir",
                            "id": "#wf_trim_and_map_pe.cwl/A_map_repeats/genomeDir"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_sort_trimmed_fastq/output_fastqsort_sortedfastq",
                            "id": "#wf_trim_and_map_pe.cwl/A_map_repeats/readFilesIn"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/A_map_repeats/aligned",
                        "#wf_trim_and_map_pe.cwl/A_map_repeats/output_map_unmapped_fwd",
                        "#wf_trim_and_map_pe.cwl/A_map_repeats/output_map_unmapped_rev",
                        "#wf_trim_and_map_pe.cwl/A_map_repeats/starsettings",
                        "#wf_trim_and_map_pe.cwl/A_map_repeats/mappingstats"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/A_map_repeats"
                },
                {
                    "run": "#fastqsort.cwl",
                    "scatter": "#wf_trim_and_map_pe.cwl/A_sort_repunmapped_fastq/input_fastqsort_fastq",
                    "in": [
                        {
                            "source": [
                                "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r1/outfile",
                                "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r2/outfile"
                            ],
                            "id": "#wf_trim_and_map_pe.cwl/A_sort_repunmapped_fastq/input_fastqsort_fastq"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/A_sort_repunmapped_fastq/output_fastqsort_sortedfastq"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/A_sort_repunmapped_fastq"
                },
                {
                    "run": "#fastqsort.cwl",
                    "scatter": "#wf_trim_and_map_pe.cwl/A_sort_trimmed_fastq/input_fastqsort_fastq",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/X_trim_again/output_trim",
                            "id": "#wf_trim_and_map_pe.cwl/A_sort_trimmed_fastq/input_fastqsort_fastq"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/A_sort_trimmed_fastq/output_fastqsort_sortedfastq"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/A_sort_trimmed_fastq"
                },
                {
                    "run": "#barcodecollapse_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/X_sortlexico/output_sort_bam",
                            "id": "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe/input_barcodecollapsepe_bam"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe/output_barcodecollapsepe_bam",
                        "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe/output_barcodecollapsepe_metrics"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe"
                },
                {
                    "run": "#sort.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/X_barcodecollapsepe/output_barcodecollapsepe_bam",
                            "id": "#wf_trim_and_map_pe.cwl/X_sort/input_sort_bam"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/X_sort/output_sort_bam"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/X_sort"
                },
                {
                    "run": "#namesort.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/rename_mapped_genome/outfile",
                            "id": "#wf_trim_and_map_pe.cwl/X_sortlexico/input_sort_bam"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/sort_names",
                            "id": "#wf_trim_and_map_pe.cwl/X_sortlexico/name_sort"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/X_sortlexico/output_sort_bam"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/X_sortlexico"
                },
                {
                    "run": "#trim_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/trim_error_rate",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/error_rate"
                        },
                        {
                            "source": [
                                "#wf_trim_and_map_pe.cwl/read1",
                                "#wf_trim_and_map_pe.cwl/read2"
                            ],
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/input_trim"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/get_A_adapters/output",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/input_trim_A_adapters"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/get_a_adapters/output",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/input_trim_a_adapters"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/get_g_adapters/output",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/input_trim_g_adapters"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/trimfirst_overlap_length",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/input_trim_overlap_length"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/trim_times",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim/times"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/X_trim/output_trim",
                        "#wf_trim_and_map_pe.cwl/X_trim/output_trim_report"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/X_trim"
                },
                {
                    "run": "#trim_pe.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/trim_error_rate",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/error_rate"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/X_trim/output_trim",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/input_trim"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/get_A_adapters/output",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/input_trim_A_adapters"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/get_a_adapters_default/output",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/input_trim_a_adapters"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/get_g_adapters_default/output",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/input_trim_g_adapters"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/trimagain_overlap_length",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/input_trim_overlap_length"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/trim_times",
                            "id": "#wf_trim_and_map_pe.cwl/X_trim_again/times"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/X_trim_again/output_trim",
                        "#wf_trim_and_map_pe.cwl/X_trim_again/output_trim_report"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/X_trim_again"
                },
                {
                    "run": "#file2stringArray.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_adapters",
                            "id": "#wf_trim_and_map_pe.cwl/get_A_adapters/file"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/get_A_adapters/output"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/get_A_adapters"
                },
                {
                    "run": "#file2stringArray.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/a_adapters",
                            "id": "#wf_trim_and_map_pe.cwl/get_a_adapters/file"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/get_a_adapters/output"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/get_a_adapters"
                },
                {
                    "run": "#file2stringArray.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/a_adapters_default",
                            "id": "#wf_trim_and_map_pe.cwl/get_a_adapters_default/file"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/get_a_adapters_default/output"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/get_a_adapters_default"
                },
                {
                    "run": "#file2stringArray.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/g_adapters",
                            "id": "#wf_trim_and_map_pe.cwl/get_g_adapters/file"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/get_g_adapters/output"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/get_g_adapters"
                },
                {
                    "run": "#file2stringArray.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/g_adapters_default",
                            "id": "#wf_trim_and_map_pe.cwl/get_g_adapters_default/file"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/get_g_adapters_default/output"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/get_g_adapters_default"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/read1",
                            "valueFrom": "${ return self.nameroot + \".genome-mapped\"; }",
                            "id": "#wf_trim_and_map_pe.cwl/rename_mapped_genome/newname"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_map_genome/aligned",
                            "id": "#wf_trim_and_map_pe.cwl/rename_mapped_genome/srcfile"
                        },
                        {
                            "default": ".bam",
                            "id": "#wf_trim_and_map_pe.cwl/rename_mapped_genome/suffix"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/rename_mapped_genome/outfile"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/rename_mapped_genome"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/read1",
                            "valueFrom": "${ return self.nameroot + \".repeat-mapped\"; }",
                            "id": "#wf_trim_and_map_pe.cwl/rename_mapped_repeats/newname"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_map_repeats/aligned",
                            "id": "#wf_trim_and_map_pe.cwl/rename_mapped_repeats/srcfile"
                        },
                        {
                            "default": ".bam",
                            "id": "#wf_trim_and_map_pe.cwl/rename_mapped_repeats/suffix"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/rename_mapped_repeats/outfile"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/rename_mapped_repeats"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/read1",
                            "valueFrom": "${ return self.nameroot + \".repeat-unmapped\"; }",
                            "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r1/newname"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_map_repeats/output_map_unmapped_fwd",
                            "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r1/srcfile"
                        },
                        {
                            "default": ".fq",
                            "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r1/suffix"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r1/outfile"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r1"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/read2",
                            "valueFrom": "${ return self.nameroot + \".repeat-unmapped\"; }",
                            "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r2/newname"
                        },
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_map_repeats/output_map_unmapped_rev",
                            "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r2/srcfile"
                        },
                        {
                            "default": ".fq",
                            "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r2/suffix"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r2/outfile"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/rename_unmapped_repeats_r2"
                },
                {
                    "run": "#wf_fastqc.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim/gzipped",
                            "valueFrom": "${\n  return self[0];\n}\n",
                            "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R1/reads"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R1/output_qc_report",
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R1/output_qc_stats"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R1"
                },
                {
                    "run": "#wf_fastqc.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim/gzipped",
                            "valueFrom": "${\n  return self[1];\n}\n",
                            "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R2/reads"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R2/output_qc_report",
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R2/output_qc_stats"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_R2"
                },
                {
                    "run": "#wf_fastqc.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again/gzipped",
                            "valueFrom": "${\n  return self[0];\n}\n",
                            "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R1/reads"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R1/output_qc_report",
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R1/output_qc_stats"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R1"
                },
                {
                    "run": "#wf_fastqc.cwl",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again/gzipped",
                            "valueFrom": "${\n  return self[1];\n}\n",
                            "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R2/reads"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R2/output_qc_report",
                        "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R2/output_qc_stats"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_fastqc_trim_again_R2"
                },
                {
                    "run": "#gzip.cwl",
                    "scatter": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim/input",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/X_trim/output_trim",
                            "id": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim/input"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim/gzipped"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim"
                },
                {
                    "run": "#gzip.cwl",
                    "scatter": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again/input",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_sort_trimmed_fastq/output_fastqsort_sortedfastq",
                            "id": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again/input"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again/gzipped"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_gzip_sort_X_trim_again"
                },
                {
                    "run": "#gzip.cwl",
                    "scatter": "#wf_trim_and_map_pe.cwl/step_gzip_sort_repunmapped_fastq/input",
                    "in": [
                        {
                            "source": "#wf_trim_and_map_pe.cwl/A_sort_repunmapped_fastq/output_fastqsort_sortedfastq",
                            "id": "#wf_trim_and_map_pe.cwl/step_gzip_sort_repunmapped_fastq/input"
                        }
                    ],
                    "out": [
                        "#wf_trim_and_map_pe.cwl/step_gzip_sort_repunmapped_fastq/gzipped"
                    ],
                    "id": "#wf_trim_and_map_pe.cwl/step_gzip_sort_repunmapped_fastq"
                }
            ],
            "doc": "This workflow takes in appropriate trimming params and demultiplexed reads,\nand performs the following steps in order: trimx1, trimx2, fastq-sort, filter repeat elements, fastq-sort, genomic mapping, sort alignment, index alignment, namesort, PCR dedup, sort alignment, index alignment\n",
            "id": "#wf_trim_and_map_pe.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
